<?php
/**
 * This file provides different helper functions intended to be
 * used through ajax calls.
 */

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

require_once('classes/common_gui.php');
require_once('include/changelog.inc.php');

$action = false;
$data = false; # because that's the best way to say "Error: invalid request"
$retval = 0;
$obj_type = false;

if (isset($_REQUEST['action'])) {
	$action = trim(addslashes($_REQUEST['action']));
}

switch ($action) {
	case 'object_data':
		$obj_id = false;
		$obj_name = false;
		if (isset($_REQUEST['obj_type'])) {
			$obj_type = trim($_REQUEST['obj_type']);
		}

		$obj_type = str_replace('new_', '', $obj_type);
		if (isset($_REQUEST['obj_id'])) {
			$obj_id = $_REQUEST['obj_id'];
		}
		if (isset($_REQUEST['obj_name'])) {
			$obj_name = $_REQUEST['obj_name'];
		}
		$valid_obj_types = array('host', 'service', 'contact', 'host_template', 'service_template', 'contact_template');

		if (!$obj_type || !in_array($obj_type, $valid_obj_types)) {
			header("HTTP/1.0 400 Bad request");
			echo "Can not resolve object type ($obj_type) or object id ($obj_id)";
			ob_end_flush();
			exit(1);
		}

		$class_name = $obj_type . '_nacoma_Model';
		$obj = new $class_name($obj_id);
		if ($obj_name) {
			$obj->get_object_by_name($obj_name);
		}
		if (!$obj->id) {
			$obj->get_default_object();
		}
		$obj->expand_object();
		$data = $obj->denormalize();
		break;

	case 'command_info':

		$command_data_dir = "/opt/plugins/commandinfo/";
		$data = array();

		if ( isset( $_REQUEST[ 'plugin' ] ) ) {

			$plugin = $_REQUEST[ 'plugin' ];
			$path = $command_data_dir . $plugin . ".json";

			if ( is_readable( $path ) ) {
				$content = file_get_contents( $path );
				$data = json_decode( $content );
			} else {
				$data[ "error" ] = "No information";
			}

		} else {

			$data[ "error" ] = "No plugin given";

		}

		break;

	case 'object_list':
		if ( isset( $_REQUEST['object_type'] ) ) {
			$data = array();
			$type = $_REQUEST['object_type'];
			$class = $type . '_nacoma_Model';
			$tmp = $class::get_objects_for_user();

			foreach ( $tmp as $id => $name ) {
				$data[] = $name;
			}
		}
		break;

	case 'hosts_with_svc':
		require_once(dirname(__FILE__) . '/include/data.inc.php');
		if (isset($_REQUEST['service']) && !empty($_REQUEST['service'])) {
			$svc = trim($_REQUEST['service']);
			$sql = "SELECT DISTINCT host.host_name FROM host INNER JOIN service ON service.host_name = host.id WHERE service.service_description = '".$svc."'";
			$res = nacoma_db::exec_query($sql);
			if ($res != false && count($res) > 0) {
				while ($row = $res->fetch(PDO::FETCH_ASSOC)) {
					$data[] = $row['host_name'];
				}
			}
		}
		break;

	/**
	 * Takes hosts, sends back services allowed, i.e. those
	 * who have graph data available
	 */
	case 'pnp_page_services_for_hosts':
		if(!isset($_GET['hosts']) || !$_GET['hosts']) {
			// do not run expensive runs if we don't want to
			$data = array();
			break;
		}
		$available_graphs = pnp_page_nacoma_Model::get_available_graphs();
		$hosts_to_examine = $_GET['hosts'];

		// @todo accept sent in services? benchmark
		$available_services = array();
		foreach($available_graphs as $host => $graph) {
			if(in_array($host, $hosts_to_examine)) {
				$services = array_keys($graph);
				$available_services = array_merge($available_services, $services);
			}
		}
		$data = $available_services;
		break;

	case 'extend_lock':
		if (!isset($_REQUEST['obj_type']) || empty($_REQUEST['obj_type']))
			break;
		if (!isset($_REQUEST['obj_id']) || empty($_REQUEST['obj_id']))
			break;
		$retval = nacoma_objlock::extend_lock($_REQUEST['obj_type'], $_REQUEST['obj_id']);
		$retval = $retval === true ? 1 : 0;
		break;

	case 'is_user_contact':
		require_once(dirname(__FILE__) . '/include/nacoma_access.inc.php');
		if (!isset($_REQUEST['objs']) || empty($_REQUEST['objs']))
			break;
		foreach ($_REQUEST['objs'] as $obj) {
			if (!isset($obj['obj_type']) || empty($obj['obj_type']))
				continue;
			if (isset($obj['contacts']))
				$contacts = explode(';', $obj['contacts']);
			else
				$contacts = array();
			if (isset($obj['contact_groups']))
				$cgroups = explode(';', $obj['contact_groups']);
			else
				$cgroups = array();
			$template = isset($obj['template']) ? $obj['template'] : false;
			$retval = is_user_contact($obj['obj_type'], $contacts, $cgroups, $template) ? 1 : 0;
			if ($retval == 0)
				break 2;
		}
		break;
	case 'services_for_hostgroup':
		if (!isset($_REQUEST['obj_id']))
			break;
		$hg = new hostgroup_nacoma_Model($_REQUEST['obj_id']);
		$slaves = $hg->find_slave_objects('service');
		$data = array();
		foreach($slaves as $slave) {
			$svc = new service_nacoma_Model($slave);
			$data[] = $svc->get_object_name();
		}
		break;
	case 'merlin_nodes':
		require_once('include/probe.inc.php');
		require_once('include/system.inc.php');
		$data = get_merlin_nodes();
		break;
}

echo json_encode($data);
