<?php
$base_path = dirname(__FILE__).'/../';
require_once($base_path.'include/data.inc.php');
include_once($base_path.'include/webconfig.inc.php');
include_once($base_path.'include/import.inc.php');

// ===========================================
//	API Functions
// ===========================================

/**
 * Add new or edit existing item of certain object type
 */
function edit_object($obj=false,$values=false, $new_field_name=false, $new_field_value=false)
{
	if (!is_object($obj)) {
		return false;
	}

	if (!empty($values)) {
		$props = $obj->get_properties();

		foreach ($values as $k => $value) {
			if (!isset($props['var_index'][$k]) && isset($props['deprecated_names'][$k])) {
				$final_k = $props['deprecated_names'][$k];
			}
			else if (!isset($props['var_index'][$k])) {
				$erray['Invalid variable'] = "The variable name $k was not known";
			}
			else {
				$final_k = $k;
			}

			switch ($props['var_index'][$final_k]) {
			 case 'mselect':
			 case 'access':
				$values[$final_k] = explode(',', $value);
			}
			if ($k == 'Access Levels')
				$values[$final_k] = explode(',', $value);

		}

		try {
			$obj->normalize($values);
		} catch (NacomaRelationException $ex) {
			$erray['Normalization exception'] = $ex->getMessage();
			return $erray;
		}

		if (empty($obj->obj)) {
			$obj->get_default_object();
		}
	}

	# rename?
	if (!empty($new_field_name) && !empty($new_field_value)) {
		if ($obj->obj && array_key_exists('id', $obj->obj) && (int)$obj->obj['id'] && $obj->obj['id'] !== 'new') {
			$obj->obj[$new_field_name] = $new_field_value;
		} else {
			$erray['Object not found'] = 'Unable to rename non-existing object.';
			return $erray;
		}
	}

	// Save and take care of return value
	return $obj->save_object() === true ? true : $obj->validation_error_list;
}
/**
 * Print encountered errors in a readable way to console
 */
function api_display_error($erray)
{
	global $DEBUG;
	if (!empty($erray) && $erray !== true) {
		if (is_array($erray)) {
			if ($DEBUG) {
				echo "Failed to save the object, for the following reason(s):\n";
				printf('%30s  %s', '------------------------------',
					'--------------------------------------------------'."\n");
				printf('%30s  %s', '|                 variable   |',
				'|			Reason	                 |'."\n");
				printf('%30s  %s', '------------------------------',
				'--------------------------------------------------'."\n");
				foreach ($erray as $v => $reason) {
					printf('%30s: %s', $v, $reason."\n");
				}
			} else {
				foreach ($erray as $v => $reason) {
					printf('%30s: %s', $v, $reason."\n");
				}
			}
		}
		return false;
	} elseif ($erray !== true) {
		return false;
	} else {
		return true;
	}
}

/**
 * Lock all tables to prevent concurrent db write
 */
function api_issue_table_lock()
{
    // ACHTUNG: SHOW TABLES and LOCK TABLES are not portable SQL
	$result = false;
	$sql = "SHOW TABLES FROM monitor_webconfig";
	$res = nacoma_db::exec_query($sql);
	$tables = array();
	while ($row = $res->fetch(PDO::FETCH_NUM)) {
			$tables[] = 'monitor_webconfig.'.$row[0];
	}
        unset($res);
	$lock_action = 'WRITE';
	$sql_str = " host h $lock_action, "; // aliases used
	if (!empty($tables)) {
		$sql_str .= implode(' ' . $lock_action . ', ', $tables);
		$sql_str = "LOCK TABLES " . $sql_str . " $lock_action;";
		nacoma_db::exec_query($sql_str);
	}
}

/**
 * Restore table access
 */
function api_unlock_tables()
{
	nacoma_db::exec_query("UNLOCK TABLES");
}

/**
 * Fetch objects by type and return list
 * @param $obj_type string denoting object type
 */
function api_list_objects($obj_type=false)
{
	$class = $obj_type . '_nacoma_Model';
	$obj = new $class();
	$objects = $obj::get_objects_for_user();
	$return_str = '';
	if (!empty($objects)) {
		$obj_names = array();
		foreach ($objects as $oid => $internal_name) {
			$name = get_object_name_by_id($obj_type, $oid);
			if (!$name)
				$name = $internal_name;
			if (!$obj->properties['topic'])
				$name .= " #$oid";
			$obj_names[] = $name;
		}
		$return_str = implode("\n", $obj_names);
		return $return_str;
	}
	return false;
}

/**
 * Fetch (denormalized) object properties
 */
function api_show_object($obj)
{
	if (!is_object($obj)) {
		return false;
	}
	if (empty($obj->obj))
		return false;
	$contents = $obj->denormalize();
	$obj_list = false;
	if (!empty($obj->id) && $obj->id != 'new') {
		foreach ($contents as $key => $val) {
			$value = is_array($val) ? implode(",", $val) : $val;
			$obj_list[] = $key . '=' . $value;
		}
		return implode("\n", $obj_list)."\n";
	}
	return false;
}
