<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

require_once('include/webconfig.inc.php');
require_once('include/nacoma_access.inc.php');

function apply_custom_vars($obj, $q) {
	if( isset($q['custom_vars']) ) {
		$qcv = $q['custom_vars'];
	} else {
		return;
	}

	if( isset($obj->type) && isset($qcv[$obj->type]) ) {
		$qcv = $qcv[$obj->type];
	} else {
		return;
	}

	if( isset($obj->id) && isset($qcv[$obj->id]) ) {
		$qcv = $qcv[$obj->id];
	} else if( empty($obj->id) && isset($qcv['new']) ) {
		$qcv = $qcv['new'];
	} else {
		return;
	}

	$custom_vars = array();
	foreach($qcv as $pair) {
		$custom_vars[$pair['key']] = $pair['value'];
	}
	$obj->custom_vars = $custom_vars;
}

function print_escaped($text, $do_escape = true) {
	if( $do_escape )
		print htmlentities($text, ENT_QUOTES, 'UTF-8');
	else
		print $text;
}


function get_escape_html_tags() {
	global $Config;
	$lines = file($Config['cgi']);
	if( $lines === false )
		return null;
	foreach($lines as $line) {
		if( preg_match(':[\s]*escape_html_tags[\s]*=[\s]*([^\s]+)[\s]*:', $line, $parts) ) {
			return 0 != $parts[1];
		}
	}
	return true; /* Default: escape */
}

header('Cache-Control: no-cache');

function check_csrf() {
	if ((PHP_SAPI === 'cli') || (!in_array($_SERVER['REQUEST_METHOD'], array('POST', 'PUT', 'DELETE')))) {
		return;
	}
	$csrf_header = 'X-op5-csrf-token';

	$log = op5log::instance('nacoma');
	$log->log('debug', 'Validating CSRF token');
	$headers = getallheaders();
	if (
		(!isset($_REQUEST['csrf_token']) || !Session::instance()->csrf_token_valid($_REQUEST['csrf_token']))
		&&
		(!array_key_exists($csrf_header, $headers) || !Session::instance()->csrf_token_valid($headers[$csrf_header]))
	) {
		$log->log('warning', 'CSRF token validation failed');
		header('HTTP/1.1 403 Forbidden');
		echo "Mismatching CSRF tokens, you might need to login to op5 Monitor again";
		exit(1);
	}

	$log->log('debug', 'CSRF token validation successful');
}

check_csrf();

$q = $_POST;

$obj = array();
$cmd_node = false;
if (!empty($q['cmdTestNode']))
	$cmd_node = $q['cmdTestNode'];

$do_escape = get_escape_html_tags();

$host = false;
$service = false;
$command = false;
$runcmd = false;
$objtype = false;

if (isset($q['obj_type']) && $q['obj_type'] == 'service') {
	$access = get_nacoma_access('test_this_service');
	if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		exit('You are not allowed to test this service.');
	}
	$service = new service_nacoma_Model($q['tmp_obj_id']);
	$service->normalize($q[$q['page_selection']][$q['tmp_obj_id']]);
	apply_custom_vars($service, $q);
	$service->collapse_object();
	$service->template_expand_object();

	$host_id = false;
	if( isset($q['master_obj_id']) ) {
		$host_id = intval($q['master_obj_id']);
	} else {
		$host_id = $service->master_obj_id;
	}

	if( $host_id !== false ) {
		$host = new host_nacoma_Model($host_id);
		apply_custom_vars($host, $q);
		$host->template_expand_object();
	}

	$command_id = $service->obj['check_command'];
	$command = new command_nacoma_Model($command_id);
	$objtype = 'service';
}
if (isset($q['obj_type']) && $q['obj_type'] == 'host') {
	$access = get_nacoma_access('test_this_host');
	if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		exit('You are not allowed to test this host.');
	}
	$host = new host_nacoma_Model(intval($q['tmp_obj_id']));
	$host->normalize($q[$q['page_selection']][$q['tmp_obj_id']]);
	apply_custom_vars($host, $q);
	$host->collapse_object();
	$host->template_expand_object();

	$command_id = $host->obj['check_command'];
	$command = new command_nacoma_Model($command_id);
	$objtype = 'host';
}
if (isset($q['cmd_str'])) {
	$access = get_nacoma_access('test_this_command');
	if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
		exit('You are not allowed to test this command.');
	}
	$command = new command_nacoma_Model();
	$command->obj['command_line'] = $q['cmd_str'];
	$command->obj['command_name'] = 'test_command';
	$objtype = 'command';
}

$cmdrun = new cmd_test_nacoma_Model();

if( $command !== false ) $cmdrun->attach_object( $command );
if( $service !== false ) $cmdrun->attach_object( $service );
if( $host    !== false ) $cmdrun->attach_object( $host );

try {
	print_escaped($cmdrun->cmdline($objtype)."\n", true);
	print_escaped($cmdrun->runcmd($objtype, $cmd_node), $do_escape);
} catch( op5queryhandler_Exception $e ) {
	if( $DEBUG ) {
		print $e;
	} else {
		print $e->getMessage();
	}
}
