<?php
/*
 * Copyright(C) 2003-2005, 2007 op5 AB
 * All rights reserved.
 *
 */
require_once('system.inc.php');
require_once('config.inc.php');
require_once('data.inc.php');

$import_warnings = array();
$import_errors = array();
$index = array();
$num_objs = array();
$obj_array = array();
$custom_vars = array();

function get_last_import_time()
{
	// REMINDER: UNIX_TIMESTAMP() is mysql-specific!
	$query = 'SELECT UNIX_TIMESTAMP(time) AS time FROM action_log ' .
		"WHERE action = 'import' OR action = 'export' " .
		'ORDER BY time DESC LIMIT 1';
	$result = nacoma_db::exec_query($query);
	return ($row = $result->fetch(PDO::FETCH_ASSOC))
		? $row['time']
		: 0;
}

function changes_since_last_import()
{
	global $HOME;
	clearstatcache();

	$last_import_time = get_last_import_time();
	if (!$last_import_time)
		return true;

	$old_files = array();
	$res = nacoma_db::exec_query('SELECT file_name FROM file_tbl');
	$tmp = $res->fetchAll(PDO::FETCH_NUM);

	// Workaround: through weird actions and bad luck, we could end up with a
	// last import time, but no imported data - this should force an import in
	// such cases
	if (empty($tmp))
		return true;

	foreach ($tmp as $file)
		$old_files[] = $file[0];
	$cfg_files = get_config_files();
	foreach ($old_files as $file) {
		if ($file[0] != '/')
			$file = $HOME . $file;
		if (!in_array($file, $cfg_files))
			$cfg_files[] = $file;
	}
	foreach($cfg_files as $cfg_file) {
		$mtime = @filemtime($cfg_file);
		if ($mtime > $last_import_time || $mtime === false)
			return true;
	}

	return(false);
}

function make_dir_p($dir)
{
	if (is_dir($dir))
		return (true);

	$ary = explode('/', $dir);
	$path = '';
	foreach ($ary as $dirpart) {
		$path .= '/' . $dirpart;
		if (is_dir($path))
			continue;

		if (!mkdir($path))
			return false;
	}

	return true;
}

function is_valid_timeperiod_timerange($str)
{
	# if it contains chars that can't possibly be in a
	# timeperiod range, it can't be a match
	if (preg_match("/[^0-9-:, \t]/", $str))
		return false;

	# if at least the first entry doesn't look like a timeperiod,
	# it can't be a match either
	if (!preg_match('/^[012][0-9]:[0-5][0-9]/', $str))
		return false;

	# passed two (rather basic) tests, so it's probably ok
	return true;
}

# we don't bother with services, since those are 1-to-M with
# hosts. As such, they'll get new ID's later
function import_get_object_name($obj_type, $obj)
{
	if (isset($obj['name']))
		return false;

	if (isset($obj["${obj_type}_name"]))
		return $obj["${obj_type}_name"];

	return false;
}

function import_error_handler($errno, $errstr, $errfile, $errline, $errcontext)
{
	if (!$errcontext || !isset($errcontext['i']) || !isset($errcontext['fname']) || !isset($errcontext['line']))
		return; // can't print anything useful anyway :(
	echo "<p>There was a problem importing your configuration on line {$errcontext['i']} in file {$errcontext['fname']}: \"$errstr\". The line reads:</p>";
	echo "<pre>{$errcontext['line']}</pre>";
	$l = op5Log::instance('nacoma');
	$l->log('error', "There was a problem importing your configuration on line {$errcontext['i']} in file {$errcontext['fname']}: \"$errstr\". The line reads: {$errcontext['line']}");
}

/**
 * Parse objects from a single object config file.
 * Optimized for precision, not speed.
 */
function import_objects($fname)
{
	global $HOME;
	global $num_objs, $obj_array;
	global $custom_vars;

	set_error_handler('import_error_handler');

	$tp = new timeperiod_nacoma_Model();
	$tp_index = $tp->properties['var_index'];
	$inobj = false;
	$objs = 0;

	if (!isset($num_objs['FILE']))
		$num_objs['FILE'] = 1;

	# we know name and id, so create the FILE entries ad-hoc
	# This must come before we try to read the file, or
	# zero-length files will not show up in the gui
	idx_set('FILE', $fname, $num_objs['FILE']++);
	$file_id = idx_get('FILE', $fname);
	$obj_cfg_file = str_replace($HOME, '', $fname);
	$obj_array['FILE'][$file_id]['file_name'] = $obj_cfg_file;

	if(!($buf = read_file($fname)))
		return false;

	$lines = explode("\n", $buf);

	$obj = array();
	$i = -1;
	foreach($lines as $line) {
		$i++;
		# strip end-of-line comments before trim()'ing
		$line = preg_replace('/(?<!\\\\);.*$/', '', $line);

		# Skip comments and empty lines
		$line = trim($line);
		if (!strlen($line) || $line{0} === '#')
			continue;

		/* add object type */
		if(!strncmp($line, 'define', strlen('define'))) {
			$str = preg_split("/[\t {]+/", $line, 3);
			$obj_type = $str[1];
			if (!isset($num_objs[$obj_type]))
				$num_objs[$obj_type] = 1;
		}

		if($line{strlen($line) - 1} === '{') {
			$inobj = true;
			$id = $num_objs[$obj_type]++;
			continue;
		}
		if ($line{0} === '}') {
			if ($obj_type === 'host' || $obj_type === 'service') {
				# remove the deprecated checks_enabled variable
				# in favour of the {passive,active}_checks_enabled
				# ones instead
				if (isset($obj['checks_enabled'])) {
					if (!isset($obj['active_checks_enabled']))
						$obj['active_checks_enabled'] = $obj['checks_enabled'];
					if (!isset($obj['passive_checks_enabled']))
						$obj['passive_checks_enabled'] = $obj['checks_enabled'];
					unset($obj['checks_enabled']);
				}
				# Nagios 3 deprecates 'normal_check_interval' and
				# 'retry_check_interval' in favour of 'check_interval'
				# and 'retry_interval', respectively
				if (isset($obj['normal_check_interval'])) {
					$obj['check_interval'] = $obj['normal_check_interval'];
					unset($obj['normal_check_interval']);
				}
				if (isset($obj['retry_check_interval'])) {
					$obj['retry_interval'] = $obj['retry_check_interval'];
					unset($obj['retry_check_interval']);
				}
				if (isset($obj['obsess_over_service'])) {
					$obj['obsess'] = $obj['obsess_over_service'];
					unset($obj['obsess_over_service']);
				}
				if (isset($obj['obsess_over_host'])) {
					$obj['obsess'] = $obj['obsess_over_host'];
					unset($obj['obsess_over_host']);
				}

				# We don't swallow "failure_prediction_enabled", but
				# nagios does. We remove it unconditionally to prevent
				# barfing when we try to write to database later on
				unset($obj['failure_prediction_enabled']);
				# We deleted this variable in nacoma, because it served
				# little-to-no purpose.
				# bug #7443
				unset($obj['initial_state']);
				# vrml is silly and nobody uses it and we don't have DB columns for it
				unset($obj['vrml_image']);
				unset($obj['3d_coords']);
			}

			# 'use' is a reserved word in mysql, so translate to 'template'
			if (isset($obj['use'])) {
				$obj['template'] = $obj['use'];
				unset($obj['use']);
			}

			# {host,service}_notification_commands* have too long names for
			# some databases, so we rename them
			if ($obj_type == 'contact') {
				if (isset($obj['service_notification_commands'])) {
					$obj['service_notification_cmds'] = $obj['service_notification_commands'];
					unset($obj['service_notification_commands']);
				}
				if (isset($obj['service_notification_commands_args'])) {
					$obj['service_notification_cmds_args'] = $obj['service_notification_commands_args'];
					unset($obj['service_notification_commands_args']);
				}
				if (isset($obj['host_notification_commands'])) {
					$obj['host_notification_cmds'] = $obj['host_notification_commands'];
					unset($obj['host_notification_commands']);
				}
				if (isset($obj['host_notification_commands_args'])) {
					$obj['host_notification_cmds_args'] = $obj['host_notification_commands_args'];
					unset($obj['host_notification_commands_args']);
				}
			}

			# Nagios 3 adds {execution,notification}_failure_options
			# as aliases for {execution,notification}_failure_criteria
			# for both dependency type objects
			if ($obj_type === 'hostdependency' || $obj_type == 'servicedependency') {
				if (isset($obj['notification_failure_options'])) {
					$obj['notification_failure_criteria'] = $obj['notification_failure_options'];
					unset($obj['notification_failure_options']);
				}
				if (isset($obj['execution_failure_options'])) {
					$obj['execution_failure_criteria'] = $obj['execution_failure_options'];
					unset($obj['execution_failure_options']);
				}
			}

			$obj['file_id'] = $file_id;

			if (isset($obj['name'])) {
				idx_set($obj_type . '_template', $obj['name'], $id);
			}

			if ($obj_type !== 'FILE' && $obj_type !== 'service' && $name = import_get_object_name($obj_type, $obj)) {
				idx_set($obj_type, $name, $id);
			}
			elseif ($obj_type === 'service') {
				// Preserve the id given at config parse to match custom vars for services.
				$obj['id'] = $id;
				# Nacoma only handles the underscore-free variation of this, but
				# nagios considers both legal.
				if (isset($obj['service_groups'])) {
					$obj['servicegroups'] = $obj['service_groups'];
					unset($obj['service_groups']);
				}
			}


			$obj_array[$obj_type][$id] = $obj;
			$obj = array();
			continue;
		}

		if(!$inobj)
			continue;

		$str = preg_split("/[\t ]+/", $line, 2);
		$k = $str[0];
		$v = isset($str[1]) ? $str[1] : '';
		if ($k{0} === '_') {
			# custom variable
			$custom_vars[$obj_type][$id][$k] = $v;
			unset($obj[$k]);
			continue;
		}
		if ($obj_type === 'timeperiod' && $k != 'timeperiod_name'
			&& $k != 'alias' && $k != 'exclude' &&
			!is_valid_timeperiod_timerange($v))
		{
			# almost certainly an exception
			$ary = preg_split("/[\t ]+/", $line);
			$v = '';
			$k = '';
			while ($part = array_pop($ary)) {
				if (is_valid_timeperiod_timerange($part))
					$v[] = trim($part);
				else {
					# put the non-matching entry back
					array_push($ary, $part);
					break;
				}
			}
			$v = join(',', $v);
			$k = join(' ', $ary);
		}

		if ($obj_type === 'timeperiod' && !isset($tp_index[$k])) {
			# definitely an exception. Stash it with the custom vars
			$custom_vars[$obj_type][$id][$k] = $v;
			continue;
		}

		# not a timeperiod
		$obj[$k] = $v;
	}

	restore_error_handler();

	return $obj_array;
}

# Recursively locates all files matching *.cfg under the specified path
function glob_cfg_dir($directory, &$cfg_files)
{
	$dirhandle = opendir($directory);
	if (!$dirhandle)
		return false;

	while (($dirent = readdir($dirhandle)) !== false) {
		if ($dirent{0} === '.')
			continue;

		if (is_dir($directory . '/' . $dirent)) {
			glob_cfg_dir($directory . '/' . $dirent, $cfg_files);
			continue;
		}

		if (is_file($directory . '/' . $dirent) &&
			substr($dirent, -4, 4) === '.cfg')
		{
			$cfg_files[] = $directory . '/' . $dirent;
			continue;
		}
	}

	closedir($dirhandle);

	return true;
}

/*
 * Get configuration files from main config file and return them in an array.
 */
function get_config_files($nagios_cfg=false) {
	global $Config, $import_warnings, $import_errors;
	global $notouch_files, $notouch_file_prefix, $notouch_dir_prefix;

	if (!$nagios_cfg)
		$nagios_cfg = $Config['main'];

	$cfg_files = array();

	// read, mangle and put in array
	$buf = read_file($nagios_cfg);
	$rel_base = dirname($nagios_cfg).'/';
	$lines = explode("\n", $buf);
	foreach($lines as $line) {
		// skip junk, split string and evaluate
		if(!strlen($line) || $line[0] === '#') continue;
		$str = preg_split("/[\t ]*=[\t ]*/", trim($line), 2);
		if($str[0] == "cfg_file") {
			if ($str[1][0] != '/')
				$str[1] = $rel_base . $str[1];
			$cfg_files[] = $str[1];
		}
		# found a dir, so get files from it recursively
		else if($str[0] == "cfg_dir") {
			$dir = $str[1];
			if (is_string($notouch_dir_prefix)) {
				# translates '/foo/var/boo///' to 'boo' as well
				$dir = basename($dir);
				if (substr($dir, 0, strlen($notouch_dir_prefix)) === $notouch_dir_prefix) {
					continue;
				}
			}
			if ($str[1][0] != '/')
				$str[1] = $rel_base . $str[1];
			glob_cfg_dir($str[1], $cfg_files);
		}
	}

	# weed out files we aren't supposed to touch
	$prefix_len = 0;
	if (is_string($notouch_file_prefix))
		$prefix_len = strlen($notouch_file_prefix);
	foreach ($cfg_files as $k => $f) {
		if (is_array($notouch_files) && in_array($f, $notouch_files)) {
			unset($cfg_files[$k]);
			continue;
		}

		if (!$prefix_len)
			continue;

		if (substr(basename($f), 0, $prefix_len) === $notouch_file_prefix)
			unset($cfg_files[$k]);
	}

	# make sure we can read and write all the files we're importing from
	$grp_info = posix_getgrgid(posix_getegid());
	$grp_name = $grp_info['name'];
	foreach ($cfg_files as $f) {
		$solution = "To correct this problem, log in as root to your system " .
					"and run the following commands:\n\n" .
					"    chown monitor:$grp_name $f\n" .
					"    chmod 664 $f\n\n";
		if (posix_access($f, POSIX_W_OK) === false) {
			$import_warnings[] = "I don't have write access to '$f'.\n" .
				"Trying to save configuration without write permissions " .
				"on all configuration files will fail.\n" . $solution;
		}
		if (posix_access($f, POSIX_R_OK) === false) {
			$import_errors[] = "I don't have read access to '$f'.\n" .
				"Trying to import configuration without read permissions " .
				"on all files will fail.\n" . $solution;
		}
	}
	return($cfg_files);
}

global $master_ary;
$master_ary =
	array('host' =>
		  array('service' => 1, 'hostdependency' => 1,
				'hostescalation' => 1),
		  'service' =>
		  array('serviceescalation' => 1, 'servicedependency' => 1)
		  );

function import_get_targets($obj_type, $target_type, &$obj, $k)
{
	global $master_ary;
	global $obj_array;

	assert("!is_array(\$obj['$k'])");

	$ary = import_explode($obj[$k]);
	$ary_size = count($ary);
	$ret = $exclude = array();
	for ($i = 0; $i < $ary_size; $i++) {
		$target = $ary[$i];
		if ($target_type === 'service' && $k === 'members')
			$target .= ',' . $ary[++$i];

		$tid = idx_get($target_type, $target);
		$ret[$tid] = $tid;
	}

	return $ret;
}

# adds a new (unique) value to the index
function idx_set($obj_type, $name, $id)
{
	global $index, $DEBUG;

	if (isset($index[$obj_type][$name])) {
		echo "index mismatch: Duplicate entry for $obj_type object '$name'\n";
		if ($DEBUG) {
			echo "<pre>\n"; debug_print_backtrace(); echo "</pre>\n";
		}
		$l = op5Log::instance('nacoma');
		$l->log('warning', "Index mismatch: Duplicate entry for $object_type object '$name'");
		$l->log('debug', "Index mismatch found at ".implode("\n", debug_backtrace()));
		exit(1);
	}
	$index[$obj_type][$name] = $id;
}

# fetches a value from the index, ensuring it exists
# $playnice: if true and something fails, don't scream and die
function idx_get($obj_type, $name, $playnice=false)
{
	global $index, $DEBUG;

	//remove the additive symbol if present
	$name = preg_replace('/^\+/', '', $name, 1);

	if (empty($index[$obj_type][$name])) {
		if ($playnice)
			return false;
		echo "index mismatch: No index entry for '$obj_type' object '$name'\n";
		if ($DEBUG) {
			echo "<pre>\n"; debug_print_backtrace(); echo "</pre>\n";
		}
		$l = op5Log::instance('nacoma');
		$l->log('warning', "Index mismatch: No index entry for $obj_type object '$name'");
		$msg = "";
		foreach (debug_backtrace() as $item) {
			$msg .= implode("\n", $item);
		}
		$l->log('debug', "Index mismatch found at $msg");
		exit(1);
	}
	return $index[$obj_type][$name];
}

function get_ary_val($ary, $val)
{
	if (!isset($ary[$val]))
		return false;
	return $ary[$val];
}

function import_explode($str)
{
	assert('!is_array($str)');
	if ($str !== false)
		return preg_split("/[\t ]*,[\t ]*/", $str, -1, PREG_SPLIT_NO_EMPTY);

	return false;
}

function import_get_master_list($master, $groups, $objects)
{
	global $obj_array;
	$exc_groups = $inc_groups = $exc_objs = $inc_objs = array();

	assert('is_array($groups) || $groups === false');
	assert('is_array($objects) || $objects === false');
	assert('$objects !== false || $groups !== false');
	assert('!empty($objects) || !empty($groups)');
	assert("'$master' === 'service' || '$master' === 'host'");

	$ret = array($master.'group' => array(), $master => array());
	if ($objects !== false) {
		foreach ($objects as $obj_name) {
			if ($obj_name{0} === '!') {
				$name = substr($obj_name, 1);
				$exc_objs[$name] = $name;
				continue;
			}

			$inc_objs[$obj_name] = $obj_name;
		}
	}
	if ($groups !== false) {
		foreach ($groups as $group_name) {
			if ($group_name[0] === '!') {
				$group_name = substr($group_name, 1);
				$exc_groups[$group_name] = $group_name;
				$id = idx_get($master . 'group', $group_name);
				if (isset($obj_array[$master . 'group'][$id]['members'])) {
					foreach ($obj_array[$master . 'group'][$id]['members'] as $member_name) {
						$exc_objs[$member_name] = $member_name;
					}
				}
				continue;
			}
			$inc_groups[$group_name] = $group_name;
		}

		foreach ($inc_groups as $id) {
			if (!isset($exc_group[$id])) {
				$ret[$master.'group'][$id] = $id;
				$id = idx_get($master . 'group', $group_name);
				if (isset($obj_array[$master . 'group'][$id]['members'])) {
					foreach ($obj_array[$master . 'group'][$id]['members'] as $member_name) {
						$exc_objs[$member_name] = $member_name;
					}
				}
			}
		}
	}

	foreach ($inc_objs as $id) {
		if (!isset($exc_objs[$id]))
			$ret[$master][$id] = $id;
	}

	return $ret;
}

# parse a host slave object
$slave_obj_array = array();
function parse_host_slave($obj_type, &$obj, $deps)
{
	global $obj_array, $num_objs, $slave_obj_array, $custom_vars;

	unset($deps['host_name']);
	unset($deps['hostgroup_name']);
	unset($deps['dependent_host_name']);
	unset($deps['dependent_hostgroup_name']);
	unset($deps['servicegroups']);

	parse_object($obj_type, $obj, $deps);

	# handle 'servicegroups' directive for services
	# we break out the list before duplicating the objects
	$sgs = false;
	if ($obj_type === 'service' && isset($obj['servicegroups'])) {
		$sgs = import_explode($obj['servicegroups']);
		$obj['servicegroups'] = array();
		foreach ($sgs as $sg) {
			$sgid = idx_get('servicegroup', $sg);
			$obj['servicegroups'][$sgid] = $sgid;
		}
	}
	if ($obj_type === 'service' && !isset($obj['service_description']) && !isset($obj['name']) && isset($obj['template'])) {
		$obj['service_description'] = $obj_array['service'][$obj['template']]['service_description'];
	}

	if (isset($obj['name']) && isset($obj['register']) && $obj['register'] == 0)
		return $obj;

	$groups = import_explode(get_ary_val($obj, 'hostgroup_name'));
	$objects = import_explode(get_ary_val($obj, 'host_name'));

	assert('!empty($groups) || !empty($objects)');

	if ($obj_type === 'hostdependency') {
		$dep_list = import_get_master_list('host', $groups, $objects);
		assert('!empty($dep_list)');

		$groups = import_explode(get_ary_val($obj, 'dependent_hostgroup_name'));
		$objects = import_explode(get_ary_val($obj, 'dependent_host_name'));
		$master_list = import_get_master_list('host', $groups, $objects);
		assert('!empty($master_list)');
		foreach (array_keys($master_list) as $type) {
			unset($obj['dependent_'.$type.'_name']);
			unset($obj[$type.'_name']);
		}

		foreach ($master_list as $parent_type => $object_list) {
			foreach ($object_list as $master_name) {
				$master_id = idx_get($parent_type, $master_name);
				$obj['dependent_'.$parent_type.'_name'] = $master_id;
				# special case: nacoma can represent this in one object
				if (count($dep_list['host']) === 1 && count($dep_list['hostgroup']) === 1) {
					$host = array_pop($dep_list['host']);
					$hostgroup = array_pop($dep_list['hostgroup']);
					$obj['host_name'] = idx_get('host', $host);
					$obj['hostgroup_name'] = idx_get('hostgroup', $hostgroup);
					$obj_id = $obj['id'] = $num_objs[$obj_type]++;
					$slave_obj_array[$obj_type][$obj_id] = $obj;
				} else {
					foreach ($dep_list as $dep_parent_type => $dep_obj_list) {
						foreach ($dep_obj_list as $dep_name) {
							$dep_id = idx_get($dep_parent_type, $dep_name);
							$obj[$dep_parent_type.'_name'] = $dep_id;
							$obj_id = $obj['id'] = $num_objs[$obj_type]++;
							$slave_obj_array[$obj_type][$obj_id] = $obj;
						}
						unset($obj[$dep_parent_type.'_name']);
					}
				}
			}
			unset($obj['dependent_'.$parent_type.'_name']);
		}
		return $obj;
	}

	$master_list = import_get_master_list('host', $groups, $objects);
	assert('!empty($master_list)');
	assert('!empty($master_list[\'host\']) || !empty($master_list[\'hostgroup\'])');

	foreach (array_keys($master_list) as $type)
		unset($obj[$type.'_name']);

	foreach ($master_list as $parent_type => $object_list) {
		foreach ($object_list as $master_name) {
			$master_id = idx_get($parent_type, $master_name);
			$obj[$parent_type.'_name'] = $master_id;
			$obj_id = $num_objs[$obj_type]++;
			// Fetch id from import to be able to match custom vars
			if ($obj_type === 'service') {
				$custom_var_old_id = $obj['id'];
			}
			$obj['id'] = $obj_id;
			$slave_obj_array[$obj_type][$obj_id] = $obj;
			if ($obj_type === 'service') {
				if (isset($custom_vars['service'][$custom_var_old_id])) {
					$custom_vars['service'][$obj_id] = $custom_vars['service'][$custom_var_old_id];
					unset($custom_vars['service'][$custom_var_old_id]);
				}
				$obj_name = $master_name . ';' . $obj['service_description'];
				idx_set('service', $obj_name, $obj_id);
			}
		}
		unset($obj[$parent_type.'_name']);
	}

	return $obj;
}

/**
 * Flatten a service, that is, copy lots of per-host services from one
 * hostgroup service.
 */
function flatten_service($obj) {
	global $slave_obj_array;
	global $index;
	global $obj_array;
	global $num_objs;
	global $master_ary;
	// already flattened
	if (!isset($obj['hostgroup_name']))
		return;

	// preparation
	$hosts = $obj_array['hostgroup'][$obj['hostgroup_name']]['members'];
	$hg = $obj_array['hostgroup'][$obj['hostgroup_name']]['hostgroup_name'];
	$old_id = $index['service'][$hg.';'.$obj['service_description']];
	$firstnewid = $num_objs['service'];
	$num_objs['service'] += count($hosts);
	unset($obj['hostgroup_name']);

	// fix index of name => id mappings
	unset($index['service'][$hg.';'.$obj['service_description']]);
	$i = 0;
	foreach ($hosts as $host) {
		$index['service'][$host.';'.$obj['service_description']] = $firstnewid + $i;
		$i++;
	}

	// fix object map
	unset($slave_obj_array['service'][$old_id]);
	$i = 0;
	foreach ($hosts as $host) {
		$obj['host_name'] = idx_get('host', $host);
		$slave_obj_array['service'][$firstnewid + $i] = $obj;
		$i++;
	}

	// fix references from other slave objects
	foreach ($master_ary['service'] as $slave_type => $_) {
		if (!isset($slave_obj_array[$slave_type]))
			continue;
		foreach ($slave_obj_array[$slave_type] as $id => $obj) {
			if (isset($obj['service']) && $obj['service'] === $old_id) {
				unset($slave_obj_array[$slave_type][$id]);
				for ($i = 0; $i < count($hosts); $i++) {
					$obj['service'] = $firstnewid + $i;
					$obj['id'] = $num_objs[$slave_type]++;
					$slave_obj_ary[$slave_type][$obj['id']] = $obj;
				}
			}
			else if (isset($obj['dependent_service']) && $obj['dependent_service'] === $old_id) {
				unset($slave_obj_array[$slave_type][$id]);
				for ($i = 0; $i < count($hosts); $i++) {
					$obj['dependent_service'] = $firstnewid + $i;
					$obj['id'] = $num_objs[$slave_type]++;
					$slave_obj_array[$slave_type][$obj['id']] = $obj;
				}
			}
		}
	}
}


/**
 * You see these helper functions?
 *
 * You see how ugly, ungeneric and magic they are?
 *
 * That's what you get when you don't put lambdas in your language!
 */
function parse_service_slave_actually_add($obj_type, &$obj) {
	global $slave_obj_array;
	global $num_objs;
	$obj_id = $num_objs[$obj_type]++;
	$obj['id'] = $obj_id;
	$slave_obj_array[$obj_type][$obj_id] = $obj;
}

// When a serviceescalation is for a host group and the service is for a
// host in that group, or vice versa, or either one of them (but not both)
// excludes one of the hosts in the host group, idx_get fails. Scheiße.
// This method works around by flattening first the service slave and then the
// service, hoping that it'll be able to get around the problem.
function parse_service_slave_helper($master_list, $obj_type, &$obj, $sd, $rel_name = 'service', $oncompletion='parse_service_slave_actually_add($obj_type, $obj);') {
	global $obj_array, $slave_obj_array;
	foreach ($master_list['hostgroup'] as $parent_name) {
		$obj[$rel_name] = idx_get('service', "$parent_name;$sd", true);
		if ($obj[$rel_name] === false) {
			// so, service is already flattened - we can handle this trivially!
			foreach ($obj_array['hostgroup'][idx_get('hostgroup', $parent_name)]['members'] as $mid) {
				$master_list['host'][$mid] = $mid;
			}
			continue;
		}
		eval($oncompletion);
	}
	foreach ($master_list['host'] as $parent_name) {
		$obj[$rel_name] = idx_get('service', "$parent_name;$sd", true);
		if ($obj[$rel_name] === false) {
			// service isn't flattened, but needs to be.
			// if no service exists that via it's hostgroup fulfills the
			// service name, we'll crash, but then the config is (hopefully)
			// wrong.
			foreach ($obj_array['hostgroup'] as $hgid => $hgobj) {
				if (in_array($parent_name, $hgobj['members']) &&
					idx_get('service', $hgobj['hostgroup_name'].';'.$sd, true) !== false)
				{
					flatten_service($slave_obj_array['service'][idx_get('service', $hgobj['hostgroup_name'].';'.$sd)]);
					break;
				}
			}
			$obj[$rel_name] = idx_get('service', "$parent_name;$sd");
		}
		eval($oncompletion);
	}
}

# parse a service slave object
function parse_service_slave($obj_type, &$obj, $deps)
{
	$groups = import_explode(get_ary_val($obj, 'hostgroup_name'));
	$objects = import_explode(get_ary_val($obj, 'host_name'));

	if (empty($obj['service_description']) || (!$groups && !$objects))
		return false;

	parse_object($obj_type, $obj, $deps);

	$sd = $obj['service_description'];
	unset($obj['hostgroup_name']);
	unset($obj['host_name']);
	unset($obj['service_description']);

	assert('!empty($groups) || !empty($objects)');

	$prefix = '';
	if ($obj_type === 'servicedependency') {
		$dep_list = import_get_master_list('host', $groups, $objects);
		if (empty($dep_list) || empty($obj['dependent_service_description']))
			return false;
		$master_sd = $obj['dependent_service_description'];
		unset($obj['dependent_service_description']);

		$groups = import_explode(get_ary_val($obj, 'dependent_hostgroup_name'));
		$objects = import_explode(get_ary_val($obj, 'dependent_host_name'));
		if (empty($groups) && empty($objects))
			return false;
		unset($obj['dependent_host_name']);
		unset($obj['dependent_hostgroup_name']);
		$master_list = import_get_master_list('host', $groups, $objects);
		assert('!empty($master_list)');
		parse_service_slave_helper($master_list, $obj_type, $obj, $master_sd, 'dependent_service', 'parse_service_slave_helper('.var_export($dep_list, true).', $obj_type, $obj, "'.$sd.'");');
		return $obj;
	}

	$master_list = import_get_master_list('host', $groups, $objects);
	assert('!empty($master_list)');

	parse_service_slave_helper($master_list, $obj_type, $obj, $sd);

	return $obj;
}

function parse_object($obj_type, &$obj, $deps)
{
	global $master_ary;
	global $obj_array;

	if (empty($deps))
		return $obj;

	foreach ($deps as $k => $ary) {
		if (!isset($obj[$k]))
			continue;
		$multi = $ary['multi'];
		$target_type = $ary['obj'];
		assert('!is_array($target_type)');

		# XXX this means we need to parse group objects one extra time
		if ($k === 'members') {
			$obj[$k] = import_explode($obj[$k]);
		}
		elseif ($target_type === $obj_type . 'group') {
			# {host,contact,service}groups directive encountered
			$obj_name = import_get_object_name($obj_type, $obj);
			$foo = import_explode($obj[$k]);
			if (isset($obj['name'])) {
				# it's a template, so add back-references to the
				# group in question
				$real = array();
				foreach ($foo as $group_name) {
					$gid = idx_get($target_type, $group_name);
					$real[$gid] = $gid;
				}
				$obj[$k] = $real;

				# if this was a 'pure' template, we continue
				# from here. Otherwise we must add it to the
				# group as well, so future reference lookups
				# can succeed.
				if (!$obj_name)
					continue;
			}
			foreach ($foo as $group_name) {
				$gid = idx_get($target_type, $group_name);
				if (!isset($obj_array[$target_type][$gid]['members']))
					$obj_array[$target_type][$gid]['members'] = $obj_name;
				else
					$obj_array[$target_type][$gid]['members'] .= ",$obj_name";
			}
			unset($obj[$k]);
		}
		elseif ($target_type === 'command' && strpos($obj[$k], '!')) {
			# we split commands and their arguments for
			# database normalisation reasons
			$cmd_ary = explode('!', $obj[$k], 2);
			$obj[$k] = idx_get('command', $cmd_ary[0]);
			$obj[$k . '_args'] = $cmd_ary[1];
		}
		elseif (!$multi) {
			$obj[$k] = idx_get($target_type, $obj[$k]);
		}
		else {
			$obj[$k] = import_get_targets($obj_type, $target_type, $obj, $k);
		}
	}

	return $obj;
}

function recombobulate_group_members(&$obj_array)
{
	foreach (array('host', 'contact') as $obj_type) {
		$group_type = $obj_type . 'group';
		if (!isset($obj_array[$obj_type]) || !isset($obj_array[$group_type]))
			continue;
		foreach ($obj_array[$group_type] as $obj_key => $obj) {
			# nagios doesn't allow empty groups, but we import
			# them anyway so the user can fill them up
			if (!isset($obj['members']))
				continue;

			$real = array();
			foreach ($obj['members'] as $name) {
				$mid = idx_get($obj_type, $name);
				$real[$mid] = $mid;
			}
			$obj_array[$group_type][$obj_key]['members'] = $real;
		}
	}

	if (empty($obj_array['servicegroup']))
		return;

	foreach ($obj_array['servicegroup'] as $obj_key => $obj) {
		if (!isset($obj['members']))
			continue;

		$members = $obj['members'];
		$real = array();
		$i = 0;
		$ary_size = count($members);
		for ($i = 0; $i < $ary_size; $i++) {
			$name = $members[$i];
			if (strpos($name, ';') === false) {
				$n2 = $members[++$i];
				$name .= ";$n2";
			}
			$sid = idx_get('service', $name);
			$real[$sid] = $sid;
		}
		$obj_array['servicegroup'][$obj_key]['members'] = $real;
	}
}

/**
 * Recombobulate
 *   To put something back the way it was, or into proper working order
 */
function recombobulate_object_array(&$obj_array)
{
	global $master_ary, $slave_obj_array, $index;

	# once we get here, all tier 1 objects have been indexed
	# That is, objects that are not dependant on lower-tier
	# objects for their existance are in $index (timeperiod,
	# host, host_template, contact, contact_template, contactgroup,
	# hostgroup), but not servicegroup (as they depend on services).
	# Commands have no links, so they are already completed.
	#
	# In order to be capable of resolving all slave objects, we
	# must first make sure the hostgroup->members directive is
	# normalized and populated fully.
	#
	# When that is done, we parse all slave objects belonging to
	# hosts (services, hostescalations, hostdependencies).
	#
	# In the last phase we parse all slave objects belonging to
	# services (serviceescalations, servicedependencies).

	$exp_objects = Module_Manifest_Model::get('nacoma_import_objects');

	foreach ($exp_objects as $obj_type => $classname) {
		$props = get_object_properties($obj_type);
		if (!isset($props['special']))
			continue;
		if (substr($obj_type, 0, 4) === 'new_')
			continue;

		$vindex = $props['var_index'];
		foreach ($props['special'] as $k => $v) {
			if ($k === 'file_id')
				continue;

			if (!is_array($v) && isset($master_ary[$v][$obj_type]) && !strpos($obj_type, 'dependency'))
				continue;

			# skip 'statusmap_image' and 'icon_image'
			# as they're all select variables but without an object
			# link.
			if (is_array($v) && strpos($k, 'image') !== false)
				continue;

			if ($vindex[$k] === 'mselect' || $vindex[$k] === 'select' || $vindex[$k] === 'hidden') {
				$dep[$obj_type][$k]['multi'] = ($vindex[$k] === 'mselect');
				$dep[$obj_type][$k]['obj'] = $v;
			}
		}
	}

	# command and FILE objects are now completed

	# first round of post-read parsing. Walk all tier1 objects
	$parse_order = array('timeperiod', 'contact', 'contactgroup',
						 'host', 'hostgroup', 'servicegroup');

	foreach ($parse_order as $obj_type) {
		if (!isset($obj_array[$obj_type]))
			continue;

		$deps = $dep[$obj_type];
		foreach ($obj_array[$obj_type] as $obj_id => &$obj)
			parse_object($obj_type, $obj, $deps);
	}

	# now we handle host slave objects
	foreach ($master_ary['host'] as $obj_type => $discard) {
		if (!isset($obj_array[$obj_type]))
			continue;

		foreach ($obj_array[$obj_type] as $obj_id => &$obj)
			parse_host_slave($obj_type, $obj, get_ary_val($dep, $obj_type));
	}

	foreach ($master_ary['service'] as $obj_type => $discard) {
		if (!isset($obj_array[$obj_type]))
			continue;

		foreach ($obj_array[$obj_type] as $obj_id => &$srv_slave_obj) {
			parse_service_slave($obj_type, $srv_slave_obj, get_ary_val($dep, $obj_type));
		}
	}

	# Now we can fix the 'members' directive for groups
	recombobulate_group_members($obj_array);

	# services have been attached to their host objects now,
	# so we must stash the service templates temporarily
	# to avoid them getting overwritten
	$st_array = array();
	if (isset($index['service_template'])) {
		foreach ($index['service_template'] as $st_id) {
			$st_array[$st_id] =& $obj_array['service'][$st_id];
		}
	}

	# include the slave objects in the main object array
	foreach ($slave_obj_array as $obj_type => $obj_list)
		$obj_array[$obj_type] = $obj_list;

	# put the service templates back in
	foreach ($st_array as $st_id => &$obj) {
		assert('!isset($obj_array["service"][$st_id]) || $obj_array["service"][$st_id] === $obj');
		$obj_array['service'][$st_id] =& $obj;
	}
	unset($st_array);
}


/*
 * import objects from config files(uses get_config_files)
 * All objects are sorted as per object_type here. Merging, splitting and
 * cross-referencing objects is done someplace else.
 */
function import_object_configuration($cfg_files = false)
{
	global $IMPORTING;
	global $obj_array, $index, $num_objs;

	$IMPORTING = true;

	if ($cfg_files === false)
		$cfg_files = get_config_files();
	elseif (!is_array($cfg_files))
		$cfg_files = array($cfg_files);

	$num_objs = $index = $obj_array = array();

	# do a raw import of the objects and create the index
	# table for the FILE variables while we're at it
	foreach($cfg_files as $cfg_file) {
		import_objects($cfg_file);
	}

	recombobulate_object_array($obj_array);

	return $obj_array;
}

/**
 * Calculates and returns the name of all object-related tables.
 * 'custom_vars' is always in there
 */
function get_object_table_names()
{
	global $Supported_Object_Types;

	$ret = array('custom_vars' => 'custom_vars');
	foreach ($Supported_Object_Types as $obj_type) {
		$ary = get_object_properties($obj_type);

		$ret[$obj_type] = $obj_type;

		$v_index = $ary['var_index'];
		$spec = $ary['special'];
		foreach ($v_index as $k => $v) {
			if ($v === 'mselect' && !(array_key_exists($k, $spec) && is_array($spec[$k]))) {
				$jnc_table = get_junction_table_name($obj_type, $k);
				$ret[$jnc_table] = $jnc_table;
			}
		}
	}

	# fix from when we used oracle (and other database engines), where 'FILE' is
	# a reserved word.
	unset($ret['FILE']);
	$ret['file_tbl'] = 'file_tbl';

	return $ret;
}

/**
 * Truncates all object related tables (not 'changelog')
 */
function truncate_object_tables()
{
	$ret = true;
	$tables = get_object_table_names();

	foreach ($tables as $tbl)
		nacoma_db::exec_query("TRUNCATE TABLE $tbl");
}

function glue_object_array(&$obj_array)
{
	# glue_objects also updates the necessary junctions
	foreach($obj_array as $obj_type => $obj_list) {
		glue_objects($obj_list, $obj_type);
	}
}

function update_action_log($action, $user = 'console user')
{
	return nacoma_db::exec_query("INSERT INTO action_log(username, action) VALUES(".nacoma_db::quote_string($user).", 'import')");
}
/**
 * caller / coordinator for importing configuration from scratch
 */
function import_config($user = false, $cfg_files = false, &$error_message = null)
{
	global $custom_vars;
	$error_message = null;
	global $import_errors;
	$l = op5Log::instance('nacoma');
	if (!nacoma_objlock::extend_lock('all', 0, 5*60)) {
		$error_message = 'Failed to import configuration to database, another import or export operation is already in progress. Please try again.';
		$l->log('warning', $error_message);
		return false;
	}
	$l->log('debug', 'Starting a config import');
	truncate_object_tables();
	nacoma_db::exec_query('TRUNCATE TABLE changelog');

	$obj_array = import_object_configuration($cfg_files);

	if ($obj_array === false || !empty($import_errors)) {
		nacoma_db::exec_query('TRUNCATE TABLE action_log');
		$error_message = 'There were import errors, bailing out';
		$l->log('error', $error_message);
		nacoma_objlock::free_lock('all', 0);
		return false;
	}

	/* Verify that we only try to import tables we know about */
	$import_tables = Module_Manifest_Model::get('nacoma_import_objects');
	foreach($obj_array as $obj_type => $objects) {
		if(!array_key_exists($obj_type, $import_tables)) {
			$error_message = 'Configuration contained unknown object type "'.$obj_type.'"';
			$import_errors[] = $error_message;
			nacoma_objlock::free_lock('all', 0);
			return false;
		}
	}

	glue_object_array($obj_array);
	foreach ($custom_vars as $type => $objs) {
		foreach ($objs as $id => $vars) {
			glue_custom_vars($type, $id, $vars);
		}
	}

	if (!$user)
		$user = get_user();
	update_action_log('import', $user);

	nacoma_objlock::free_lock('all', 0);
	$l->log('debug', 'Config import completed');
	return true;
}
