var month_names = ["january",  "february", "march", "april",  "may", "june", "july", "august", 'september', 'october', 'november', 'december'];
var weekday_names = ["sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"];

var end_days = {
		'-1': 'Last day',
		'-2': '2nd to last day',
		'-3': '3rd to last day',
		'-4': '4th to last day',
		'-5': '5th to last day'
};

function is_correct_timeranges(str)
{
	return (/^([01]\d|2[0-4]):([0-5]\d)-([01]\d|2[0-4]):([0-5]\d)(,([01]\d|2[0-4]):([0-5]\d)-([01]\d|2[0-4]):([0-5]\d))*$/).test(str);
}

function start_of_timerange(range)
{
	return range.replace(/^(.*)-.*$/, '$1');
}
function end_of_timerange(range)
{
	return range.replace(/^.-(.*)$/, '$1');
}

function add_exception_option(index_to_replace, text, value, hours)
{
	if(index_to_replace !== undefined)
	{
		// replace select options
		$("#exc_added option:eq("+index_to_replace+")").
			val(value).
			text(text).
			attr("attr_hours", hours);
	}
	else
	{
		// add select options
		$("#exc_added").addOption(value, text);
		$("#exc_added option[value='"+value+"']").attr("attr_hours", hours);
	}
}
/*
 * Sets the selected option in any number of select-one's
 *
 * Arguments:
 * id_prefix - A prefix to the id of every select element to be affected
 * values - hash containing values of options to be set.
 *
 * The clever part is that the hash keys are the completion of
 * the select element id.
 *
 * This might sound funny, but check it out:
 * id_prefix = foo_
 * values = {   bar: 5, baz: 10}
 * Sets the selected options in foo_bar and foo_baz
 * to the options having values 5 and 10 resp.
 */
function set_selected_values(id_prefix, values)
{
	for(var id_suffix in values)
	{
		$("#"+id_prefix+id_suffix).selectOptions(""+values[id_suffix]);
	}
}

function merge_timeranges(range1, range2)
{
	var r1 = range1.split('-');
	var r2 = range2.split('-');

	var new_start = (r1[0] < r2[0] ? r1[0] : r2[0]);
	var new_end = (r1[1] > r2[1] ? r1[1] : r2[1]);
	return sprintf("%s-%s", new_start, new_end);
}
/*
 * Simple check if timeranges overlap. If they do, they are merged.
 * Assumes timeranges are entered in chronological order.
  */
function resolve_overlapping_timeranges(str)
{
	var ranges = str.split(',');
	for(var i=0 ; i<ranges.length ; i++)
	{
		if(i < (ranges.length - 1))
		{
			var current_end  = end_of_timerange(ranges[i]);
			var next_start  = start_of_timerange(ranges[i+1]);
			if(current_end <= next_start)
			{
				ranges[i] = merge_timeranges(ranges[i], ranges[i+1]);
				ranges.splice(i+1, 1);
			}
		}
	}
	return ranges.join(',');
}

/* note: row_class should start with dot (".") */
function show_rows(row_class){ $(row_class).show(); onzebrachange($(row_class).parents('table'));}
function hide_rows(row_class) { $(row_class).hide(); onzebrachange($(row_class).parents('table'));}

/*
* check if year, month and day was selected
*/
function date_has_year_month_day(y,m,d)
{
	return (y!=='' && m!=='' && d!=='');
}
/*
* check if month and day was selected
*/
function date_has_month_day(y,m,d)
{
	return (y==='' && m!=='' && d!=='');
}

/*
* check if day and nothing else was selected
*/
function date_has_only_day(y,m,d)
{
	return (y==='' && m==='' && d!=='');
}

/*
* check if no date was selected
*/
function date_is_empty(y,m,d)
{
	return (y==='' && m==='' && d==='');
}

function add_single_day_exception(index_to_replace)
{
	var offset=$("#single_weekday_offset option:selected").val();
	var name=$("#single_weekday_name option:selected").val();
	var month=$("#single_weekday_month option:selected").val();

	var hours =$("#exc_hours").val().replace(/\s/g, '');
	var freq =$("#exc_freq").val();

	var freq_str = '';

	if(freq !== '')
	{
		freq = parseInt(freq);
		if(isNaN(freq) || freq <= 1)
		{
			alert("Frequency must be positive integer.");
			return false;
		}
		if(freq > 1)
		{
			freq_str = " / " + freq;
		}
	}

	// there are currently two possible formats:
	// 1) offset weekday month
	// 2) offset weekday

	var wday_str = weekday_names[name];
	var month_str = '';

	if(month !== '')
	{
		month_str = sprintf(" %s", month_names[month-1]);
	}

	if(!is_correct_timeranges(hours))
	{
		alert("Incorrect hours entered. Correct  format is:\n"+
				"hh:mm-hh:mm[,hh:mm-hh:mm,...]");
		return false;
	}

	var value = sprintf("%s %d%s%s", wday_str, offset, month_str, freq_str);
	var text = value+ " (" + hours.replace(',',', ') + ")";

	add_exception_option(index_to_replace, text, value, hours);
	return true;
}

function add_day_range_exception(index_to_replace)
{
	var from_offset=$("#from_weekday_offset option:selected").val();
	var from_name=$("#from_weekday_name option:selected").val();
	var from_month=$("#from_weekday_month option:selected").val();

	var to_offset=$("#to_weekday_offset option:selected").val();
	var to_name=$("#to_weekday_name option:selected").val();
	var to_month=$("#to_weekday_month option:selected").val();

	var hours =$("#exc_hours").val().replace(/\s/g, '');
	var freq =$("#exc_freq").val();

	var value='';
	var freq_str = '';

	if(freq !== '')
	{
		freq = parseInt(freq);
		if(isNaN(freq) || freq <= 1)
		{
			alert("Frequency must be positive integer.");
			return false;
		}
		if(freq > 1)
		{
			freq_str = " / " + freq;
		}
	}

	// there are currently two possible formats:
	// 1) offset weekday month
	// 2) offset weekday

	if(from_month !== '')
	{
		// inherit to_month from from_month if missing
		if(to_month === '')
			to_month = from_month;

		if(from_name == to_name && from_offset == to_offset && from_month == to_month)
		{
			value = sprintf("%s %d %s%s",
				weekday_names[from_name], from_offset, month_names[from_month-1], freq_str);
		}
		else
		{
			value = sprintf("%s %d %s - %s %d %s%s",
				weekday_names[from_name], from_offset, month_names[from_month-1],
				weekday_names[to_name], to_offset, month_names[to_month-1],
				freq_str);
		}
	}
	else if(from_month === '' && to_month === '')
	{
		if(from_name == to_name && from_offset == to_offset)
		{
			value = sprintf("%s %d%s",
				weekday_names[from_name], from_offset, freq_str);
		}
		else
		{
			value = sprintf("%s %d - %s %d%s",
				weekday_names[from_name], from_offset,
				weekday_names[to_name], to_offset,
				freq_str);
		}
	}
	else
	{
		alert("Format mismatch. Either select both from and to months or neither.");
		return false;
	}

	if(!is_correct_timeranges(hours))
	{
		alert("Incorrect hours entered. Correct  format is:\n"+
				"hh:mm-hh:mm[,hh:mm-hh:mm,...]");
		return false;
	}

	var text = value+ " (" + hours.replace(',',', ') + ")";

	add_exception_option(index_to_replace, text, value, hours);
	return true;
}

function add_date_single_exception(index_to_replace)
{
	var y=$("#date_single_year option:selected").val();
	var m=$("#date_single_month option:selected").val();
	var d=$("#date_single_day option:selected").val();
	var hours =$("#exc_hours").val().replace(/\s/g, '');

	var text='';
	var value='';

	if(d === '')
	{
		if (m === '' && y === '') {
			// special case every day, month, year
			// which actually is a date range.
			// By using this approach we don't have to change anything
			// else in the validation routines, just treat the exception
			// for what it is - a date range exception
			show_date_range_fields();
			set_selected_values("date_from_", {year: '', month: '', day: 1});
			return add_date_range_exception(index_to_replace);
		} else {
			alert("Please enter day");
			return false;
		}
	}

	if(date_has_year_month_day(y,m,d))
	{
		if(d < 1)
		{
			alert("Negative days cannot be used in an absolute date");
			return false;
		}
		value = sprintf("%d-%02d-%02d", y, m, d);
	}
	else if(date_has_month_day(y,m,d))
	{
		value = sprintf("%s %d", month_names[m-1], d);
	}
	else if(date_has_only_day(y,m,d))
	{
		value = sprintf("day %d", d);
	}
	else
	{
		alert("Date has an invalid format. \n"+
			"Valid formats are:\n\n"+
			"year-month-day\n"+
			"month-day\n"+
			"day");
		return;
	}

	if(!is_correct_timeranges(hours))
	{
		alert("Incorrect hours entered. Correct  format is:\n"+
				"hh:mm-hh:mm[,hh:mm-hh:mm,...]");
		return;
	}

	text = value + " (" + hours.replace(',', ', ') + ")";
	add_exception_option(index_to_replace, text, value, hours);
}

function add_date_range_exception(index_to_replace)
{
	var from_y=$("#date_from_year option:selected").val();
	var from_m=$("#date_from_month option:selected").val();
	var from_d=$("#date_from_day option:selected").val();

	var to_y=$("#date_to_year option:selected").val();
	var to_m=$("#date_to_month option:selected").val();
	var to_d=$("#date_to_day option:selected").val();

	var hours =$("#exc_hours").val().replace(/\s/g, '');
	var freq =$("#exc_freq").val();

	var text='';
	var value='';
	var freq_str = '';

	if(freq !== '')
	{
		freq = parseInt(freq);
		if(isNaN(freq) || freq <= 1)
		{
			alert("Frequency must be positive integer.");
			return false;
		}
		if(freq > 1)
		{
			freq_str = " / " + freq;
		}
	}

	// validate date formats
	if(from_d === '')
	{
		alert("Please enter from day.");
		return false;
	}

	if((to_y !== '' || to_m !== '') && to_d === '')
	{
		alert("Please enter to day, or leave all to field empty for an exception period without end.");
		return false;
	}

	if(date_has_year_month_day(from_y,from_m,from_d))
	{
		if(date_has_year_month_day(to_y,to_m,to_d))
		{
			if(from_d < 1)
			{
				alert("Negative days cannot be used in absolute from date");
				return false;
			}

			if(to_d < 1)
			{
				alert("Negative days cannot be used in absolute to date");
				return false;
			}

			value = sprintf("%d-%02d-%02d - %d-%02d-%02d%s",
				from_y, from_m, from_d,
				to_y, to_m, to_d,
				freq_str);
		}
		else if(date_is_empty(to_y,to_m,to_d))
		{
			// handles both single dates (if no skip is given) and
			// period without end (if skip is given)
			value = sprintf("%d-%02d-%02d%s", from_y, from_m, from_d, freq_str);
		}
		else if(date_has_month_day(to_y, to_m, to_d) && to_d >= 1)
		{
			// inherit from year
			to_y = from_y;
			value = sprintf("%d-%02d-%02d - %d-%02d-%02d%s",
					from_y, from_m, from_d,
					to_y, to_m, to_d,
					freq_str);
		}
		else if(to_d !== '' && to_d >= 1)
		{
			if(to_y === '')
				to_y = from_y;

			if(to_m === '')
				to_m = from_m;

			// inherit from month
			value = sprintf("%d-%02d-%02d - %d-%02d-%02d%s",
					from_y, from_m, from_d, to_y, to_m, to_d, freq_str);
		}
		else
		{
			alert("Incorrectly formatted to date");
			return false;
		}
	}
	else if(date_has_month_day(from_y,from_m,from_d))
	{
		if(date_has_month_day(to_y,to_m,to_d))
		{
			if(from_m == to_m)
			{
				// dont print month name twice if the same
				value = sprintf("%s %02d - %02d%s",
					month_names[from_m-1], from_d, to_d,	freq_str);
			}
			else
			{
				value = sprintf("%s %02d - %s %02d%s",
					month_names[from_m-1], from_d,
					month_names[to_m-1], to_d,
					freq_str);
			}
		}
		else if(date_is_empty(to_y, to_m, to_d))
		{
			value = sprintf("%s %02d%s",
				month_names[from_m-1], from_d,
				freq_str);
		}
		else if(date_has_only_day(to_y, to_m, to_d))
		{
			// inherit from month
				value = sprintf("%s %02d - %02d%s",
					month_names[from_m-1], from_d, to_d, freq_str);
		}
		else
		{
			alert("From and to dates must be in the same format");
			return false;
		}
	}
	else if(date_has_only_day(from_y,from_m,from_d))
	{
		if(date_has_only_day(to_y,to_m,to_d))
		{
			value = sprintf("day %d - %d%s", from_d, to_d, freq_str);

			if(from_d < to_d || (to_d < 0))
				value = sprintf("day %d - %d%s", from_d, to_d, freq_str);
			else if(from_d == to_d)
				value = sprintf("day %d%s", from_d, freq_str);
			else if(from_d > to_d)
			{
				alert("To date must be greater or equal to from date");
				return false;
			}
			else
			{
				alert("From and to dates must be in the same format");
				return false;
			}
		}
		else if(date_is_empty(to_y,to_m,to_d))
		{
			// from start date until end of month
			value = sprintf("day %d - -1%s", from_d, freq_str);
		}
		else
		{
			alert("From and to dates must be in the same format");
			return false;
		}
	}
	else
	{
		// from date has an invalid format
		alert("From date has an invalid format.\n"+
			"Valid formats are:\n\n"+
			"year-month-day\n"+
			"month-day\n"+
			"day");
		return false;
	}

	if(!is_correct_timeranges(hours))
	{
		alert("Incorrect hours entered. Correct  format is:\n"+
				"hh:mm-hh:mm[,hh:mm-hh:mm,...]");
		return;
	}

	text = value+ " (" + hours.replace(',',', ') + ")";

	add_exception_option(index_to_replace, text, value, hours);
}
function setup_initial_visibility()
{
	// initially hide/show correct fields
	$('tr[class^=date_]').hide();
	$('tr[class^=day_]').hide();

	if($('#exc_date').attr('checked'))
	{
		show_rows('.date_rows');
		if($('#exc_single_date').attr('checked'))
		{
			show_rows('.date_single_rows');
		}
		else if($('#exc_date_range').attr('checked'))
		{
			show_rows('.date_range_rows');
		}
	}

	if($('#exc_day').attr('checked'))
	{
		show_rows('.day_rows');
		if($('#exc_single_day').attr('checked'))
		{
			show_rows('.day_single_rows');
		}
		else if($('#exc_day_range').attr('checked'))
		{
			show_rows('.day_range_rows');
		}
	}
}

function show_single_date_fields()
{
	$("#exc_date").attr("checked", true);
	$("#exc_single_date").attr("checked", true);

	hide_rows("tr[class^=day_]");

	show_rows(".date_rows");
	show_rows('.date_single_rows');
	hide_rows('.date_range_rows');
}

function show_date_range_fields()
{
	$("#exc_date").attr("checked", true);
	$("#exc_date_range").attr("checked", true);

	hide_rows("tr[class^=day_]");

	show_rows(".date_rows");
	hide_rows('.date_single_rows');
	show_rows('.date_range_rows');
}

function show_single_day_fields()
{
	$("#exc_day").attr("checked", true);
	$("#exc_single_day").attr("checked", true);

	hide_rows("tr[class^=date_]");

	show_rows(".day_rows");
	show_rows('.day_single_rows');
	hide_rows('.day_range_rows');
}
function show_day_range_fields()
{
	$("#exc_day").attr("checked", true);
	$("#exc_day_range").attr("checked", true);

	hide_rows("tr[class^=date_]");

	show_rows(".day_rows");
	hide_rows('.day_single_rows');
	show_rows('.day_range_rows');
}


function add_exception(event, index_to_replace)
{
	if($('#exc_date').attr("checked"))
	{
		if($('#exc_single_date').attr("checked"))
		{
			return add_date_single_exception(index_to_replace);
		}
		else if($('#exc_date_range').attr("checked"))
		{
			return add_date_range_exception(index_to_replace);
		}
		else
		{
			alert("Please choose either single date or date range as interval");
		}
	}
	else if($('#exc_day').attr("checked"))
	{
		if($('#exc_single_day').attr("checked"))
		{
			return add_single_day_exception(index_to_replace);
		}
		else if($('#exc_day_range').attr("checked"))
		{
			return add_day_range_exception(index_to_replace);
		}
	}
	else
	{
		alert("Please choose either date or day as exception type");
	}
}

function guided_to_manual() {
	var added = $('#exc_added option');
	if (added.length) {
		var adv = $('#exc_adv');
		adv.val('');
		adv = adv[0];
		added.each(function() {
			adv.value += this.value;
			adv.value += '  ';
			adv.value += $(this).attr("attr_hours");
			adv.value += "\n";
		});
	}
	hide_rows('.simple');
	$("#exc_cancel_button").click();
	show_rows('.advanced');
}

function manual_to_guided() {
	$('input[name="exc_day_interv"]:checked:visible', this.form).click();
	var lines = $('#exc_adv').val().split('\n');
	if (!lines.length)
		return;
	$("#exc_added").empty();
	for (var i = 0; i < lines.length; i++) {
		var line = lines[i];
		if (line.indexOf(';') != -1)
			var line = line.slice(0, line.indexOf(';'));
		line = line.trim();
		var ary = line.split(/[ \t]+/);
		for (var j = 0; j < ary.length; j++) {
			if (is_correct_timeranges(ary[j])) {
				var date = ary.slice(0, j).join(' ');
				var hours = ary.slice(j).join(' ');
				var text = date + ' (' + hours.replace(/, ?/, ', ') + ')';
				add_exception_option(undefined, text, date, hours);
				break;
			}
		}
	}
	hide_rows('.advanced');
	show_rows('.simple');
	$('input[name="exc_t"]:checked:visible', this.form).click();
	$('input[name="exc_r"]:checked:visible', this.form).click();
}


$(document).ready(function() {

	setup_radiobutton_onclicks();
	setup_initial_visibility();

	setup_add_button_onclick();
	setup_save_button_onclick();
	setup_cancel_button_onclick();
	setup_remove_button_onclick();
	setup_edit_button_onclick();
});

function setup_add_button_onclick()
{
// register 'add' callback
	$('#exc_add_button').click(add_exception);
}

function setup_remove_button_onclick()
{
	$("#exc_remove_button").click(function() {
		$("#exc_added").removeOption(/./, true);
		edit_exception_index = -1;

		$("#exc_save_button").hide();
		$("#exc_cancel_button").hide();
		$("#exc_add_button").show();
	});
}

function setup_save_button_onclick()
{
	$("#exc_save_button").click(function(event) {

		add_exception(event, edit_exception_index);

		edit_exception_index = -1;
		$("#exc_save_button").hide();
		$("#exc_cancel_button").hide();
		$("#exc_add_button").show();
	});
}

function setup_cancel_button_onclick()
{
	$("#exc_cancel_button").click(function() {
		edit_exception_index = -1;

		$("#exc_save_button").hide();
		$("#exc_cancel_button").hide();
		$("#exc_add_button").show();
	});
}
function setup_radiobutton_onclicks()
{
	// register onclick callbacks to hide/show the correct fields
	$('#exc_date').click(function() {

		show_rows('.date_rows');
		hide_rows('.date_single_rows, .date_range_rows');

		hide_rows('tr[class^=day_]');

		if($('#exc_single_date').attr('checked')) {
			show_rows('.date_single_rows');
		}
		else if($('#exc_date_range').attr('checked')) {
			show_rows('.date_range_rows');
		}
	});

	$('#exc_single_date').click(function() {
		show_rows('.date_single_rows');
		hide_rows('.date_range_rows');
	});

	$('#exc_date_range').click(function() {
		hide_rows('.date_single_rows');
		show_rows('.date_range_rows');
	});


	$('#exc_day').click(function() {
		hide_rows('tr[class^=date_]');

		show_rows('.day_rows');
		hide_rows('.day_single_rows, .day_range_rows');

		var row_to_show='';
		if($('#exc_single_day').attr('checked')) row_to_show = '.day_single_rows';
		if($('#exc_day_range').attr('checked')) row_to_show = '.day_range_rows';

		if(row_to_show !== '')
			show_rows(row_to_show);

	});
	$('#exc_single_day').click(function() {
		show_rows('.day_single_rows');
		hide_rows('.day_range_rows');
	});
	$('#exc_day_range').click(function() {
		hide_rows('.day_single_rows');
		show_rows('.day_range_rows');
	});

	$('#exc_simple').click(function() {
		if (!$('#exc_added').is(':visible'))
			manual_to_guided();
	});
	$('#exc_advanced').click(function() {
		if ($('#exc_added').is(':visible'))
			guided_to_manual();
	});

}

var edit_exception_index = -1;
function setup_edit_button_onclick()
{
	$("#exc_edit_button").click(function() {
		var sels = $("#exc_added option:selected");
		if(sels.length != 1)
		{
			edit_exception_index = -1;
			alert("Select single exception to edit");
			return false;
		}

		edit_exception_index = sels[0].index;

		$("#exc_add_button").hide();
		$("#exc_save_button, #exc_cancel_button").show();
		//$("#exc_remove_button").hide();
		//$(this).hide();

		var dateranges = sels.val();
		var timeranges = sels.attr('attr_hours');

		var skip='';

		if(dateranges.match(" / "))
		{
			var parts = dateranges.split(" / ");
			dateranges = parts[0];
			skip = parts[1];
		}

		$('#exc_freq').val(skip);
		$("#exc_hours").val(timeranges);

		if(dateranges.match(/ - /))
		{ // a range of some sort
			var range_parts = dateranges.split(/ - /);
			var from = range_parts[0];
			var to = range_parts[1];

			var from_parts = from.split(/ /);
			var to_parts = to.split(/ /);

			/*
			Possibilities:

			yyyy-mm-dd - yyyy-mm-dd
			mname dd - mname dd
			mname dd - dd
			day dd - dd
			wday_offset wday month - wday_offset wday month
			wday_offset wday - wday_offset wday

			*/

			if(from.match(/^\d{4}-\d{2}-\d{2}$/))
			{
				// matches full calendar date range, yyyy-mm-dd - yyyy-m-dd
				show_date_range_fields();

				from_parts = from.split(/-/);
				to_parts = to.split(/-/);

				var from_y = from_parts[0];
				var to_y = to_parts[0];

				// remove preceding zeroes
				var from_m = sprintf("%d", from_parts[1]);
				var from_d = sprintf("%d", from_parts[2]);

				// remove preceding zeroes
				var to_m = sprintf("%d", to_parts[1]);
				var to_d = sprintf("%d", to_parts[2]);

				set_selected_values("date_from_", {year: from_y, month: from_m, day: from_d});
				set_selected_values("date_to_", {year:to_y, month:to_m, day:to_d});
			}
			else if($.inArray(from_parts[0], weekday_names) >= 0 && $.inArray(to_parts[0], weekday_names) >= 0)
			{
				// matches weekday dd [mname] - weekday dd [mname]

				var from_wday = $.inArray(from_parts[0], weekday_names)+"";
				var to_wday = $.inArray(to_parts[0], weekday_names)+"";

				var from_offset = from_parts[1];
				var to_offset = to_parts[1];

				var from_month = '';
				var to_month='';

				if(isNaN(from_offset) || isNaN(to_offset))
				{
					alert("Invalid exception format, cannot edit");
					return false;
				}

				if(from_parts.length > 2 && $.inArray(from_parts[2], month_names) >= 0 &&
					to_parts.length > 2 && $.inArray(to_parts[2], month_names) >= 0)
				{
					from_month = sprintf("%d", $.inArray(from_parts[2], month_names) + 1);
					to_month = sprintf("%d", $.inArray(from_parts[2], month_names) + 1);
				}

				show_day_range_fields();

				set_selected_values("from_weekday_", {offset: from_offset, name: from_wday, month: from_month});
				set_selected_values("to_weekday_", {offset: to_offset, name: to_wday, month: to_month});
			}
			else if($.inArray(from_parts[0], month_names) >= 0)
			{
				// matches mname day - [mname] day

				var from_month = sprintf("%d", $.inArray(from_parts[0], month_names) + 1);
				var to_month = sprintf("%d", $.inArray(to_parts[0], month_names) + 1);
				var to_day_idx = 1;
				if (to_month == "0") {
					to_month = from_month;
					to_day_idx = 0;
				}

				var from_day = sprintf("%d", from_parts[1]);
				var to_day = sprintf("%d", to_parts[to_day_idx]);

				if(isNaN(from_day) || isNaN(to_day))
				{
					alert("Invalid exception format, cannot edit");
					return false;
				}

				show_date_range_fields();

				set_selected_values("date_from_", {year:'', month:from_month, day:from_day });
				set_selected_values("date_to_", {year:'', month:to_month, day:to_day });
			}
			else if(from_parts[0] == 'day' && !isNaN(parseInt(to_parts[0])))
			{
				show_date_range_fields();

				var from_day = sprintf("%d", from_parts[1]);
				var to_day = sprintf("%d", to_parts[0]);

				set_selected_values("date_from_", {year:'', month:'', day:from_day });
				set_selected_values("date_to_", {year:'', month:'', day:to_day });
			}
			else
			{
				// unhandled format
				alert("Invalid exception format, cannot edit");
				return false;
			}
		}
		else // single
		{
			/*
			Possibilities:

			yyyy-mm-dd
			mname dd
			day dd
			wday_offset wday month
			wday_offset wday

			*/

			// FIXME: hantera "calendardate med skip, utan slut" korrekt

			var dateranges_parts = dateranges.split(/ /);

			// note: these should not be used without proper testing
			var weekday_index = $.inArray(dateranges_parts[0], weekday_names);
			var weekday_offset  = parseInt(dateranges_parts[1]);

			if(dateranges.match(/^\d{4}-\d{2}-\d{2}$/))
			{
				// matches yyyy-mm-dd

				show_single_date_fields();

				dateranges_parts = dateranges.split(/-/);
				var y = dateranges_parts[0];
				// remove preceding zeroes
				var m = sprintf("%d", dateranges_parts[1]);
				var d = sprintf("%d", dateranges_parts[2]);

				set_selected_values("date_single_", {year:y, month:m, day:d});
			}
			else if($.inArray(dateranges_parts[0], month_names) >= 0)
			{
				// matches monthname dd

				show_single_date_fields();

				var y = '';
				var m = sprintf("%d", $.inArray(dateranges_parts[0], month_names) + 1);
				var d = dateranges_parts[1];

				set_selected_values("date_single_", {year:y, month:m, day:d});

			}
			else if(dateranges_parts[0] == 'day')
			{
				// Handle day dd

				show_single_date_fields();

				var d = sprintf("%d", dateranges_parts[1]);

				set_selected_values("date_single_", {year:'', month:'', day:d});
			}
			else if(!isNaN(weekday_offset) && weekday_index >= 0)
			{
				// Handle wday_offset wday and wday_offset wday month
				var weekday_month = '';
				if(dateranges_parts.length > 2 && $.inArray(dateranges_parts[2], month_names) >= 0)
				{
					weekday_month = sprintf("%d", $.inArray(dateranges_parts[2], month_names) + 1);
				}
				show_single_day_fields();
				set_selected_values("single_weekday_", {offset:weekday_offset, name: weekday_index, month:weekday_month});
			}
		}
	});
}


/*
 * Loops through all added exception options and
 * adds them as custom_vars hidden input fields.
 *
 * These are named like
 * custom_vars[timeperiod][obj_id][x][key]
 * custom_vars[timeperiod][obj_id][x][value]
 * Where x is a sequence number.
 */
function timeperiod_add_custom_vars(f)
{
	var obj_id = f.tmp_obj_id.value;

	// add hidden inputs
	var i=0;
	// if the user pressed save while adding manual timeperiods, parse them with the standard method before saving them
	if (!$("#exc_added:visible").length)
		manual_to_guided();
	$("#exc_added option").each(function() {

		var name_prefix = sprintf("custom_vars[timeperiod][%s][%d]", obj_id, i);

		var custom_key_var = sprintf("<input type='hidden' name='%s%s' value='%s' />",  name_prefix, '[key]', $(this).val());
		var custom_value_var = sprintf("<input type='hidden' name='%s%s' value='%s' />",  name_prefix, '[value]', $(this).attr("attr_hours"));

		$("form[name=" + f.name + "]").append(custom_key_var);
		$("form[name=" + f.name + "]").append(custom_value_var);

		i++;
	});

	return true;
}

function sprintf( ) {
    // http://kevin.vanzonneveld.net
    // +   original by: Ash Searle (http://hexmen.com/blog/)
    // + namespaced by: Michael White (http://getsprink.com)
    // +    tweaked by: Jack
    // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)s
    // *     example 1: sprintf("%01.2f", 123.1);
    // *     returns 1: 123.10

    var regex = /%%|%(\d+\$)?([-+#0 ]*)(\*\d+\$|\*|\d+)?(\.(\*\d+\$|\*|\d+))?([scboxXuidfegEG])/g;
    var a = arguments, i = 0, format = a[i++];

    // pad()
    var pad = function(str, len, chr, leftJustify) {
        var padding = (str.length >= len) ? '' : Array(1 + len - str.length >>> 0).join(chr);
        return leftJustify ? str + padding : padding + str;
    };

    // justify()
    var justify = function(value, prefix, leftJustify, minWidth, zeroPad) {
        var diff = minWidth - value.length;
        if (diff > 0) {
            if (leftJustify || !zeroPad) {
                value = pad(value, minWidth, ' ', leftJustify);
            } else {
                value = value.slice(0, prefix.length) + pad('', diff, '0', true) + value.slice(prefix.length);
            }
        }
        return value;
    };

    // formatBaseX()
    var formatBaseX = function(value, base, prefix, leftJustify, minWidth, precision, zeroPad) {
        // Note: casts negative numbers to positive ones
        var number = value >>> 0;
        prefix = prefix && number && {'2': '0b', '8': '0', '16': '0x'}[base] || '';
        value = prefix + pad(number.toString(base), precision || 0, '0', false);
        return justify(value, prefix, leftJustify, minWidth, zeroPad);
    };

    // formatString()
    var formatString = function(value, leftJustify, minWidth, precision, zeroPad) {
        if (precision !== null) {
            value = value.slice(0, precision);
        }
        return justify(value, '', leftJustify, minWidth, zeroPad);
    };

    // finalFormat()
    var doFormat = function(substring, valueIndex, flags, minWidth, _, precision, type) {
        if (substring == '%%') return '%';

        // parse flags
        var leftJustify = false, positivePrefix = '', zeroPad = false, prefixBaseX = false;
        var flagsl = flags.length;
        for (var j = 0; flags && j < flagsl; j++) switch (flags.charAt(j)) {
            case ' ': positivePrefix = ' '; break;
            case '+': positivePrefix = '+'; break;
            case '-': leftJustify = true; break;
            case '0': zeroPad = true; break;
            case '#': prefixBaseX = true; break;
        }

        // parameters may be null, undefined, empty-string or real valued
        // we want to ignore null, undefined and empty-string values
        if (!minWidth) {
            minWidth = 0;
        } else if (minWidth == '*') {
            minWidth = +a[i++];
        } else if (minWidth.charAt(0) == '*') {
            minWidth = +a[minWidth.slice(1, -1)];
        } else {
            minWidth = +minWidth;
        }

        // Note: undocumented perl feature:
        if (minWidth < 0) {
            minWidth = -minWidth;
            leftJustify = true;
        }

        if (!isFinite(minWidth)) {
            throw new Error('sprintf: (minimum-)width must be finite');
        }

        if (!precision) {
            precision = 'fFeE'.indexOf(type) > -1 ? 6 : (type == 'd') ? 0 : void(0);
        } else if (precision == '*') {
            precision = +a[i++];
        } else if (precision.charAt(0) == '*') {
            precision = +a[precision.slice(1, -1)];
        } else {
            precision = +precision;
        }

        // grab value using valueIndex if required?
        var value = valueIndex ? a[valueIndex.slice(0, -1)] : a[i++];

        switch (type) {
            case 's': return formatString(String(value), leftJustify, minWidth, precision, zeroPad);
            case 'c': return formatString(String.fromCharCode(+value), leftJustify, minWidth, precision, zeroPad);
            case 'b': return formatBaseX(value, 2, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
            case 'o': return formatBaseX(value, 8, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
            case 'x': return formatBaseX(value, 16, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
            case 'X': return formatBaseX(value, 16, prefixBaseX, leftJustify, minWidth, precision, zeroPad).toUpperCase();
            case 'u': return formatBaseX(value, 10, prefixBaseX, leftJustify, minWidth, precision, zeroPad);
            case 'i':
            case 'd': {
                        var number = parseInt(+value);
                        var prefix = number < 0 ? '-' : positivePrefix;
                        value = prefix + pad(String(Math.abs(number)), precision, '0', false);
                        return justify(value, prefix, leftJustify, minWidth, zeroPad);
                    }
            case 'e':
            case 'E':
            case 'f':
            case 'F':
            case 'g':
            case 'G':
                        {
                        var number = +value;
                        var prefix = number < 0 ? '-' : positivePrefix;
                        var method = ['toExponential', 'toFixed', 'toPrecision']['efg'.indexOf(type.toLowerCase())];
                        var textTransform = ['toString', 'toUpperCase']['eEfFgG'.indexOf(type) % 2];
                        value = prefix + Math.abs(number)[method](precision);
                        return justify(value, prefix, leftJustify, minWidth, zeroPad)[textTransform]();
                    }
            default: return substring;
        }
    };

    return format.replace(regex, doFormat);
}
