<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

	require_once('classes/common_gui.php');
	require_once( "classes/plugin.php" );

	$page->html_start("Monitor Plugins");
	nacoma_objlock::die_if_locked_hard();

	$term = false;
	if (isset($_GET['search']) && strlen( $_GET["search"] ) > 1  ) {
		$term = $_GET['search'];
	}

	Plugin::load_config_commands();
	Plugin::load_plugin_data();

	/**
	 * Calcaulates the relevance of a search term against a string
	 * or array of strings
	 */
	function relevance ( $term, $subject, $weight ) {

		if ( !isset( $subject ) ) return 0;

		if ( is_array( $subject ) ) {

			$relevance = 0;

			foreach ( $subject as $item ) {
				$relevance += relevance( $term, $item, $weight );
			}

			return $relevance;

		} else {

			$full_match = 0;
			if ( strtolower( $term ) == strtolower( $subject ) )
				$full_match = 100;

			$terms = array();
			foreach( explode( " ", $term ) as $t) {
				$terms[] = preg_quote($t, '/');
			}
			$terms = "/(" . implode( "|", $terms ) . ")/i";
			$match = preg_match_all( $terms, $subject, $matches );

			$similarity = 0;

			if ( is_array( $terms ) ) {
				foreach ( $terms as $t ) {
					$similarity += similar_text( $subject, $t );
				}
			}

			$rel = ( $match * $weight );
			return round( $rel + ( $similarity / 50 ) ) + $full_match;

		}

	}

	/**
	 * Using above relevance method; this method adds up the relevance
	 * of a plugin compared to a searchterm cased on many fields with
	 * different weights
	 */
	function calculate_relevance ( $plugin, $term ) {

		$plugin->relevance += relevance($term, $plugin->name, 8);
		$plugin->relevance += relevance($term, $plugin->categories, 4);
		$plugin->relevance += relevance($term, $plugin->tags, 6);
		$plugin->relevance += relevance($term, $plugin->info, 2);
		$plugin->relevance += relevance($term, $plugin->why, 1);
		$plugin->relevance += relevance($term, $plugin->how, 3);

		foreach ( $plugin->commands as $name => $command ) {

			$cterm = preg_replace( "/check\_/i", "", $term );
			$plugin->relevance += relevance( $cterm, $name, 4);

			if ( isset( $command[ "tag" ] ) ) {
				$plugin->relevance += relevance( $term, $command[ "tag" ], 6 );
			}

		}

	}

	$plugins = array();
	$customs = array();

	foreach ( Plugin::$METADATA as $metadata ) {

		$plugin = new Plugin( $metadata );

		# Print files with no categories or category - either one might be found

		$file_parts	= explode( '/', $metadata );
		$file_name	= array_pop( $file_parts );
		$metadata	= str_replace( '.metadata', '', $file_name );

		if ($term !== false) {

			calculate_relevance( $plugin, $term );

			if ( $plugin->relevance >= 1 )
				$plugins[] = $plugin;

		} else {

			$plugins[] = $plugin;

		}

	}


	foreach ( Plugin::$CUSTOMS as $pluginsource ) {

		$plugin = new Plugin( );

		$path = pathinfo( $pluginsource, PATHINFO_FILENAME );
		$pluginsource = ltrim( $pluginsource, '.' );
		$plugin->name = $path . " ( ". $pluginsource .") ";
		$output = array();

		$customs[] = $plugin;

	}

?>

		<link href="css/pluginsearch.css" rel="stylesheet" type="text/css" />

		<script>

			$( document ).ready( function () {

				$( 'input[name="search"]' ).focus();

				$( "tr" ).bind("click", function ( e ) {

					var row = $(e.currentTarget),
						target = row.find( ".readmore" );

					if ( target.length > 0 ) {

						var parent = target.parent(),
							plugin = parent.find(".plugin"),
							info = parent.find(".additional-information");

						if ( info.css("display") === "block" ) {
							target.html( "+" );
							info.hide( 200, function () {
								plugin.css( "height", "18px" );
							} );
							row.removeClass( "expanded" );
						} else {
							target.html( "-" );
							info.show( 200 );
							row.addClass( "expanded" );
							plugin.css( "height", "auto" );
						}

					}

				});

			} );

		</script>

		<form method="get" action="plugin_search.php" style="height: 26px; padding: 4px">
			<h2 style="display: inline-block; padding: 3px 0;"> op5 Monitor Plugins </h2>
			<input type="text" name="search" placeholder="Search" value="<?php echo ($term !== false) ? html::specialchars($term) : ""; ?>" />
			<input type="submit" value="Search" />
			<?php
			if($term !== false) {
				$plugin_count = count($plugins);
				$results = $plugin_count == 1 ? 'plugin' : 'plugins';
				echo "&nbsp; &nbsp; <strong>" . $plugin_count . "</strong> " . $results . " matching '" . html::specialchars($term). "'";
				unset($results);
			}
			?>
			&nbsp; &nbsp; <a target="_BLANK" href="https://kb.op5.com/x/b4AK">More about Plugins</a>
		</form>

		<div class="section" style="height: 66%;">
		<form>

			<?php

				/**
				 * A usort filter that will sort a Plugin
				 * object array by the relevance member
				*/
				function sort_by_relevance ($a, $b) {
					return ( $a->relevance > $b->relevance ) ? -1 : 1;
				}

				if(!empty($plugins)) {

					usort( $plugins , "sort_by_relevance" );

					echo "<table>";

					echo "<colgroup>";

						echo "<col style=\"width: 6%;\" />";
						echo "<col style=\"width: 6%\" />";
						echo "<col style=\"width: 6%\" />";
						echo "<col style=\"width: 60%\" />";
						echo "<col style=\"width: 14%\" />";

					echo "</colgroup>";

					echo "<thead><tr>";

						echo "<th>Support</th>";
						echo "<th>Plugin</th>";
						echo "<th>Categories</th>";
						echo "<th>Information</th>";
						echo "<th>Platforms</th>";

					echo "</tr></thead><tbody>";

					foreach ($plugins as $plugin) {

						echo "<tr>";

						$kb_url = "https://kb.op5.com/x/bQhS";
						echo "<td class=\"support-" . $plugin->supportlevel . "\"><span><a target=\"_BLANK\" href=\"$kb_url\">$plugin->support</a></span></td>";
						echo "<td><b>" . $plugin->name . "</b></td>";

						echo "<td>" . implode( ", ", array_map(array('html', 'specialchars'), $plugin->categories)) . "</td>";

						echo "<td>";

							echo "<span class='readmore' title='Expand and read more' data-plugin='".html::specialchars($plugin->name)."'>+</span>";
							echo "<div class=\"plugin\">";

								$match = preg_match_all( "/^\s*\-.+?(\n|$)/m", $plugin->info, $matches );

								if ( $match ) {

									$matches = $matches[0];
									$first = $matches[0];
									$last = $matches[ count( $matches ) - 1 ];

									$plugin->info = preg_replace( "/$first/", "<ul>" . $first, $plugin->info );
									$plugin->info = preg_replace( "/$last/", $last . "</ul>", $plugin->info );

									foreach ( $matches as $item ) {
										$plugin->info = preg_replace( "/$item/", "<li>" . preg_replace( "/(^\s*\-|\n)/", "", $item ) . "</li>", $plugin->info );
									}

								}

								echo nl2br(html::specialchars($plugin->info));

								echo "<div class=\"additional-information\">";

									if ( is_array( $plugin->tags ) && !empty( $plugin->tags ) ) {
										echo "<b>Tags</b> : " . implode( ", ", array_map(array('html', 'specialchars'), $plugin->tags)) . "<br />";
									}

									echo "<b>Why?</b> : " . html::specialchars($plugin->why). "<br />";
									echo "<b>How?</b> : " . html::specialchars($plugin->how). "<br />";

									if ( isset($plugin->commands) && count( $plugin->commands ) ) {

										echo "<br /><b>Commands</b><ul class=\"commands\">";

										foreach ( $plugin->commands as $check => $command ) {

											$link = sprintf( Plugin::$COMMAND_URL, $check );
											$line = rawurlencode( $command[ "line" ] );

											echo "<li>";

											/*if ( $command[ "state" ] == "new" ) {
												echo ' <input type="checkbox" value="' . $line . '" name="new_command[' . $check . ']" /> '. $check .' ( <strong>Add new command</strong> ) <br />';
											} elseif ( $command[ "state" ] == "mod" ) {
												echo ' <input type="checkbox" value="' . $line . '" name="new_command[' . $check . ']" /> '. $check .' ( <strong>Update command</strong> ) <br />';
											} else {*/
												echo "<a target=\"_BLANK\" href=\"$link\">".html::specialchars($check)."</a>";
											//}

											if ( isset( $command["tag"] ) ) {
												echo " (" . html::specialchars($command["tag"]). ")";
											}

											echo "</li>";

										}

										echo "</ul>";

									}

								echo "</div>";
							echo "</div>";
						echo "</td>";

						echo "<td>" . html::specialchars($plugin->platforms). "</td>";
						echo "</tr>";

					}

					echo "</tbody></table>";

				}
			?>
		</form>
		</div>

		<div style="height: 26px;">
			<h2 style="padding: 4px 8px;"> Custom Plugins</h2>
		</div>
		<div class="section" style="height: 25%;">

			<?php
				usort( $customs , "sort_by_relevance" );

				echo "<table>";
				echo "<colgroup>";

					echo "<col style=\"width: 30%\" />";
					echo "<col style=\"width: 70%\" />";

				echo "</colgroup>";

				echo "<thead><tr>";
					echo "<th>Support</th>";
					echo "<th>Plugin</th>";
				echo "</tr></thead><tbody>";

				foreach ($customs as $plugin) {

					echo "<tr>";

						echo "<td class=\"support-" . $plugin->supportlevel . "\"><span>" . $plugin->support . "</span></td>";
						echo "<td><b>" . $plugin->name . "</b></td>";

					echo "</tr>";

				}

				echo "</tbody></table>";
			?>
		</div>

<?php
	$page->html_end();
