<?php
/**
 * Combined graphs lets you create graphs that combines multiple checks into
 * one single graph.
 *
 * This is different from Graph collections, which
 * combines several graphs into one page.
 *
 * Select the services and hosts you wish to combine into one graph below -
 * remember to consider the scale of the output.
 *
 * For more information, see http://docs.pnp4nagios.org/pnp-0.6/tpl_special
 */
class combined_graph_nacoma_Model extends pnp_nacoma_Model {
	/** TODO: Oh joy, hardcoded paths */
	private static $special_dir = "/opt/monitor/op5/pnp/templates.special/";
	private $graphs = array();
	public $properties = array(
		'var_index' => array(
			'title' => 'string',
			'comment' => 'string',
			'type' => 'select',
			'services' => 'mselect',
		),
		'required' => array(
			'title',
			'comment',
			'type',
			'services',
		),
		'label' => array(
			'title' => 'Title',
			'comment' => 'Comment',
			'type' => 'Type',
			'services' => 'Services',
		),
		'special' => array(
			'type' => array('area' => 'Area', 'line1' => 'Thin line', 'line2' => 'Medium line', 'line3' => 'Thick line'),
			'services' => array(), // to be filled with objects by get_properties
		),
		'topic' => 'title',
	);
	const TYPE = 'combined_graph';

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.combined_graphs";
	}

	public function __construct($id = false) {
		$this->type = static::TYPE;
		$this->get_properties();
		if (!$id)
			return;

		if (!$this->get_object_by_name($id))
			$this->id = false;
	}

	function get_object_by_name($name)
	{
		$name = basename($name, '.php');
		$this->id = $name;
		return $this->get_object();
	}

	private static function get_filename($name)
	{
		return self::$special_dir . static::objname_to_pnp($name) . '.php';
	}

	function get_properties()
	{
		$out = $this->get_available_graphs();
		$this->properties['special']['services'] = array_merge($out['hosts'], $out['services']);
		asort($this->properties['special']['services']);
		return $this->properties;
	}

	static public function object_exists($id)
	{
		return file_exists(self::get_filename($id));
	}

	function get_object() {
		$filename = self::get_filename($this->id);
		if (!is_writable($filename)) {
			return false;
		}
		$file = file($filename);
		# If nacoma didn't generate it, then I can't touch this
		if (strpos($file[1], '#NACOMA') === false)
			return false;

		foreach ($file as $line) {
			$row = explode('=', $line, 2);
			if (count($row) !== 2)
				continue;
			list($name, $value) = $row;
			$name = ltrim($name,'$');
			switch($name) {
				case 'title':
				case 'type':
				case 'comment':
				case 'services':
					eval($line);
					$this->obj[$name] = ${$name};
					break;
				case 'service':
					eval($line);
					break;
				case 'hosts':
					# old template: hosts separately, always below main code,
					# only one service, discovered last iteration
					eval($line);
					$this->obj['services'] = array();
					foreach ($hosts as $host) {
						$this->obj['services']["$host;$service"] = "$host;$service";
					}
					break;
			}
		}
		return $this->obj;
	}

	function delete_object() {
		if(!$this->id) {
			$this->delete_message = "Couldn't find combined graph";
			return false;
		}
		$file = $this->get_filename($this->id);
		if(!file_exists($file)) {
			$this->delete_message = "The file '".$file."' does not exist";
			return false;
		}
		if(!is_writable($file)) {
			$this->delete_message = "I don't have sufficient rights for removing the file ".$file;
			return false;
		}
		if(!unlink($file)) {
			$this->delete_message = "Tried to remove '$file' and failed even though the permissions are correct";
			return false;
		}

		return true;
	}

	/**
	 * @return boolean
	 */
	public function banish_object() {
		if (!$this->delete_object())
			return false;
		$this->delete_message = array('obj' => array());
		return true;
	}

	function get_cmd_bar_links() {
		$links = parent::get_cmd_bar_links();
		unset($links['propagate']); // It uses db object lookup, and is of questionable value
		return $links;
	}

	public static function get_objects_for_user($write = false)
	{
		$ret = array();
		$configs = opendir(self::$special_dir);
		while ($fname = readdir($configs)) {
			$graph = fopen(self::$special_dir.$fname, 'r');
			$head = fread($graph, 13);
			if (strpos($head, '#NACOMA') !== false)
				$ret[$fname] = $fname;
			fclose($graph);
		}
		closedir($configs);
		return $ret;
	}

	function get_object_name() {
		return isset($this->obj['title']) ? $this->obj['title'] : 'New Combined Graph';
	}

	function save_object($user=true) {
		if(!$this->obj['title']) {
			return $this->validation_error('invalid object', "Couldn't find object");
		}
		if ($this->id && $this->obj['title'] !== $this->id && $this->id != 'new') {
			if (!$this->delete_object()) {
				return $this->validation_error('permission', "Couldn't delete previous file name {$this->id}, refusing to write new copy with other name: " . $this->delete_message);
			}
		}

		$content = '<?php'."\n";
		$content.= "#NACOMA Generated - Don't edit by hand.\n";
		$content.= "\$title=\"{$this->obj['title']}\";\n";
		$content.= "\$comment=\"{$this->obj['comment']}\";\n";
		$content.= "\$type=\"{$this->obj['type']}\";\n";
		$content.= "\$services=array(";
		foreach ($this->obj['services'] as $service) {
			$content.= "'$service',";
		}
		$content = substr($content, 0, -1);
		$content.= ");\n";
		$content.= "include(\"include/special.inc.php\");\n";

		$content.= '?>'."\n";
		// Removing this makes my syntax highlighting sad :(
		?><?php

		$full_path = $this->get_filename($this->obj['title']);
		if (!file_put_contents($full_path, $content, LOCK_EX)) {
			$this->validation_error_list[self::$special_dir] = "unwritable";
			return false;
		}

		$this->id = $this->obj['title'];
		return true;
	}
}
