<?php defined('SYSPATH') OR die('No direct access allowed.');
/**
 * Configuration controller used to connect to Nacoma
 * Requires authentication
 *
 *  op5, and the op5 logo are trademarks, servicemarks, registered servicemarks
 *  or registered trademarks of op5 AB.
 *  All other trademarks, servicemarks, registered trademarks, and registered
 *  servicemarks mentioned herein may be the property of their respective owner(s).
 *  The information contained herein is provided AS IS with NO WARRANTY OF ANY
 *  KIND, INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY, AND FITNESS FOR A
 *  PARTICULAR PURPOSE.
 */
class Configuration_Controller extends Ninja_Controller {

	public function __construct () {

		parent::__construct();

		$toolbar_definition = Module_Manifest_Model::get('toolbar');
		$toolbar = new Toolbar_Controller(_("Configuration"));

		if (isset($toolbar_definition['manage'])) {
			foreach ($toolbar_definition['manage'] as $key => $def) {
				$toolbar->{$def['type']}($def['icon'], $def['label'], array(
					"href" => call_user_func_array(array('url', 'method'), $def['href'])
				));
			}
		}

		$this->template->title = _('Configuration » Configure');
		$this->template->toolbar = $toolbar;

	}

	private function last_import_time () {

		$query = 'SELECT UNIX_TIMESTAMP(time) AS time FROM action_log ' .
			"WHERE action = 'import' OR action = 'export' " .
			'ORDER BY time DESC LIMIT 1';

		$result = nacoma_db::exec_query($query);
		return ($row = $result->fetch(PDO::FETCH_ASSOC)) ? $row['time'] : 0;

	}

	public function index () {

		$this->_verify_access('ninja.configuration:read');
		$mayi = op5MayI::instance();

		if (!$this->last_import_time()) {
			url::redirect(url::method('configuration', 'configure', array('action' => 'Import')));
		}

		$this->template->js[] = "/monitor/op5/nacoma/js/jquery.filterable.js";
		$this->template->css[] = "/monitor/op5/nacoma/css/jquery.filterable.css";

		// TODO extract everything regarding the sitemap into a Ninja
		// "sitemap" model and use that model when generating all menus
		$sitemap = Module_Manifest_Model::get('sitemap');

		/* Filter out everything the user does not have access to */
		foreach($sitemap['manage']['children'] as $section => $properties) {
			if(!isset($properties['children'])) {
				continue;
			}
			foreach($properties['children'] as $menu_item_title => $menu_item_properties) {
				// right and right_any could easily be
				// complemented with right_none or right_all,
				// if wanted
				if (isset($menu_item_properties['right'])) {
					if(!$mayi->run($menu_item_properties['right'])) {
						unset($sitemap['manage']['children'][$section]['children'][$menu_item_title]);
					}
				}
				if (isset($menu_item_properties['right_any'])) {
					$allowed = false;
					foreach($menu_item_properties['right_any'] as $right) {
						if($mayi->run($right)) {
							$allowed = true;
							break;
						}
					}
					if(!$allowed) {
						unset($sitemap['manage']['children'][$section]['children'][$menu_item_title]);
					}
				}
			}
			if(count($sitemap['manage']['children'][$section]['children']) === 0) {
				// remove empty subsections
				unset($sitemap['manage']['children'][$section]);
			}
		}

		// Explicitly maintain the previous order of the menu's
		// headings, but only include the sections that weren't removed
		// above
		$sections = array_intersect(
			array(
				'monitoring_objects',
				'monitoring_options',
				'permissions',
				'templates',
				'graphs'
			),
			array_keys($sitemap['manage']['children'])
		);

		foreach ($sitemap['manage']['children'] as $key => $section) {
			if (!in_array($key, $sections, true)) {
				$sections[] = $key;
			}
		}

		$allowed_edit_host = array_filter(
			array_map(
				function($right) use ($mayi) {
					return $mayi->run($right);
				},
				array(
					'monitor.monitoring.hosts:create',
					'monitor.monitoring.hosts:update',
					'monitor.monitoring.hosts:delete',
				)
			)
		);

		$this->template->content = new View('index');
		$this->template->content->sections = $sections;
		$this->template->content->allowed_edit_host = $allowed_edit_host;

		$this->template->content->sitemap = $sitemap['manage']['children'];
	}

	/**
	 * Enable links from Ninja to Nacoma
	 *
	 * If 'service' is passed as GET, it will be used
	 * Checks are also made that NACOMA is configured in config file
	 *
	 * @param string $type
	 * @param string $name
	 */
	public function configure ($type=false, $name=false) {

		$this->_verify_access('ninja.configuration:read');

		$scan = $this->input->get('scan', null);
		$type = $this->input->get('type', $type);
		$name = $this->input->get('name', $name);
		$service = $this->input->get('service', false);
		$page = $this->input->get('page', false);
		$action = $this->input->get('action', false);

		if (Kohana::config('config.nacoma_path')===false) {
			return false;
		}

		$type = trim($type);
		$name = trim($name);

		$target_link = null;

		if ($page) {
			$target_link = $page;
		}

		if (!empty($type) && !empty($name)) {
			if (strstr($type, 'group')) {
				$target_link = 'edit.php?obj_type='.$type.'&obj_name='.urlencode($name);
			} else {
				if (!empty($service)) {
					$target_link = 'edit.php?obj_type='.$type.'&host='.urlencode($name).'&service='.urlencode($service);
				} else {
					$target_link = 'edit.php?obj_type='.$type.'&'.$type.'='.urlencode($name);
				}
			}
		} elseif (!empty($type)) {
			$target_link = 'edit.php?obj_type='.$type;
		} elseif (!empty($scan)) {
			$target_link = 'host_wizard.php?action='.$scan;
		} elseif ($action) {
			$target_link = '?action='.$action;
		} elseif (!$page) {
			$this->index();
			return;
		}

		$url = '/'.implode("/", array_map(
			function($p) {
				return trim($p, '/');
			},
			array(Kohana::config('config.nacoma_path'), $target_link)
		));
		$this->template->content = "<iframe src='$url' style='width: 100%; height: 768px' frameborder='0' id='iframe' class='save_config_nac_iframe'></iframe>";
		$this->template->nacoma = true;
	}
}

